package Alexandria::CDWS;
use Moose;
use namespace::autoclean;

use Catalyst::Runtime 5.80;

# TEST
#$ENV{DBIC_TRACE}=4;

# Set flags and add plugins for the application.
#
# Note that ORDERING IS IMPORTANT here as plugins are initialized in order,
# therefore you almost certainly want to keep ConfigLoader at the head of the
# list if you're using it.
#
#         -Debug: activates the debug mode for very useful log messages
#   ConfigLoader: will load the configuration from a Config::General file in the
#                 application's home directory
#  Session
#  Session::Store::DBI
#  Session::State::Cookie
#  Unicode::Encoding


use Catalyst qw/
  -Debug

  Observe

  ConfigLoader

  Static::Simple

  Authentication

  +Alexandria::Authentication::Store::DBI

/;

extends 'Catalyst';

our $VERSION = '2.9';

# Configure the application.
#
# Note that settings in alexandria-cdws.conf (or other external
# configuration file that you set up manually) take precedence
# over this when using ConfigLoader. Thus, configuration
# details given here can function as a default configuration,
# with an external configuration file acting as an override for
# local deployment.

__PACKAGE__->config(
    name => 'Alexandria-CDWS',

    # Disable deprecated behavior needed by old applications
    disable_component_resolution_regex_fallback => 1,

    # encode to utf-8
    encoding => 'utf-8',

    # set default view for RenderView
    default_view => 'HTML',

    # proxy x-forward-for if given
    using_frontend_proxy => 1,

    'Plugin::ConfigLoader' => {
        file => ($ENV{ALEXANDRIA_CDWS_ROOT}) ?
                   "$ENV{ALEXANDRIA_CDWS_ROOT}/alexandria-cdws.conf" :
                   'alexandria-cdws.conf'
    },
    'Plugin::Static::Simple' => {
      dirs => [ 'static' ]
    },
    version => $VERSION,
);

# Start the application
__PACKAGE__->setup();

=head1 NAME

Alexandria - Catalyst based application

=head1 SYNOPSIS

=head1 DESCRIPTION

The Catalyst application Alexandria::MMC serves the DWH and SOLR indexes for
Alexandria.

=head1 SEE ALSO

L<Alexandria::MMC::Controller::Root>, L<Catalyst>

=head1 AUTHOR

David Santamauro, E<lt>david.santamauro@fairviewresearch.comE<gt>

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;
