package Alexandria::CDWS::Controller::Admin::Privileges;

use Moose;
use namespace::autoclean;

BEGIN {extends 'Catalyst::Controller::REST'; }


=head1 NAME

Alexandria::Controller::Admin::Privileges - Privilege administration controller


=head1 DESCRIPTION

Catalyst Controller.

=head1 METHODS

=cut

use Data::Dumper;

sub auto :Private {
  my ($self, $c) = @_;
  my $user = $c->user;

  # $c->log->debug( __PACKAGE__ . Dumper( $user ) );

  if( !$user->member_of_group('Administrators') ) {
    $self->status_forbidden( $c, message => 'Insufficient priviledges' );
    $c->detach();
  }
  return 1;
}

sub list : Local : ActionClass('REST') { }
sub list_GET  :Args(0){
  my ( $self, $c ) = @_;

  my $params = $c->request->parameters;

  # set serializer
  $self->config->{map}->{'text/xml'}
    = 'XML::Alexandria::XResponse::Admin::Privileges::XList';

  my $ret = $c->model('Users')->privilege_list();

  # $c->log->debug( __PACKAGE__ . Dumper( $ret ) );

  if( $ret ) {
    my %content = ( status => 'success', privileges => $ret );
    return $self->status_ok( $c, entity => \%content );
  }
  return $self->status_bad_request( $c, message => 'Unknown response from server');
}

sub info : Local : ActionClass('REST') { }
sub info_GET  :Args(0){
  my ( $self, $c ) = @_;

  my $params = $c->request->parameters;
  my $id = $params->{privilege_id};

  # set serializer
  $self->config->{map}->{'text/xml'}
    = 'XML::Alexandria::XResponse::Admin::Privilieges::XInfo';

  my $ret = $c->model('Users')->privilege_info( $id );

  # $c->log->debug( __PACKAGE__ . Dumper( $ret ) );

  if( $ret ) {
    my %content = ( status => 'success', privilege => $ret );
    return $self->status_ok( $c, entity => \%content );
  }
  return $self->status_bad_request( $c, message => 'Unknown response from server');
}

=head2 end

=cut

sub serialize : ActionClass('Serialize') {}
sub end :Private {
  my ($self, $c) = @_;
  $c->forward('serialize');
}

=head1 AUTHOR

David Santamauro

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->config(
  'default'   => 'text/xml',
  'stash_key' => 'rest',
  'map'       => {

         'text/xml'           => 'XML::Simple',
         'application/json'   => 'JSON::Alexandria::JSONResponse',
         'text/x-json'        => 'JSON::Alexandria::JSONResponse',

         'text/html'          => 'YAML::HTML',
         'text/x-data-dumper' => [ 'Data::Serializer', 'Data::Dumper' ],
         'text/x-data-denter' => [ 'Data::Serializer', 'Data::Denter' ],
         'text/x-data-taxi'   => [ 'Data::Serializer', 'Data::Taxi'   ],
         'application/x-storable'   => [ 'Data::Serializer', 'Storable' ],
         'application/x-freezethaw' => [ 'Data::Serializer', 'FreezeThaw' ],
         'text/x-config-general'    => [ 'Data::Serializer', 'Config::General' ],
         'text/x-php-serialization' => [ 'Data::Serializer', 'PHP::Serialization' ],
        }
);


__PACKAGE__->meta->make_immutable;

1;
