package Alexandria::CDWS::Controller::Family;
use Moose;
use namespace::autoclean;

use JSON ();
use Data::Dumper;
use URI::Escape;
use Time::HiRes qw(gettimeofday tv_interval);

BEGIN {extends 'Catalyst::Controller::REST'; }

=head1 NAME

Alexandria::Controller::Family - Main family / inpadoc controller


=head1 DESCRIPTION

Search Catalyst Controller.

=head1 METHODS

=cut

use Data::Dumper;

sub simple : Local : ActionClass('REST') { }
sub simple_GET  :Args(0){
  my ( $self, $c ) = @_;
  my $user = $c->user;
  my $t = [gettimeofday];
  my $params = $c->request->parameters;

  my $family = $c->model( $user->rdb_model  )->family_simple( $params->{ucid} );
  my %ret = (
    status => 'success',
    time => sprintf("%.6f", tv_interval( $t ) ),
    family => $family
  );
  return $self->status_ok( $c, entity => \%ret );
}

sub extended : Local : ActionClass('REST') { }
sub extended_GET  :Args(0){
  my ( $self, $c ) = @_;
  my $user = $c->user;
  my $t = [gettimeofday];
  my $params = $c->request->parameters;
  # fudge for mssql
  $params->{compare_mssql} = 1;
  my $family = $c->model( $user->rdb_model  )->family_extended(
     $params->{ucid},
     $c->model( $user->idx_model ),
     ( $params->{include_basic} ) ? $params->{include_basic} : 0,
     ( $params->{compare_mssql} ) ? $c->model( 'MSSQL' ) : undef,
# only mssql
     0
  );
  my %ret = (
    status => 'success',
    time => sprintf("%.6f", tv_interval( $t ) ),
    family => $family
  );
  return $self->status_ok( $c, entity => \%ret );
}

=head2 end

=cut

sub serialize : ActionClass('Serialize') {}
sub end :Private {
  my ($self, $c) = @_;
  # CAUTION: unknown side affects. Somehow, a POST
  # request, regardless of content-type and response->code,
  # sets the body to "Content-Type xxx is not supported"
  # For successul request/response, undef this because the
  # serializer sets the body
  if( $c->response->code == 200 && $c->response->has_body ) {
    $c->response->body(undef);
  }
  $c->forward('serialize');
}


=head1 AUTHOR

David Santamauro

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->config(
  'default'   => 'application/json',
  'stash_key' => 'rest',
  'map'       => {

         'text/xml'           => 'XML::Simple',
         'application/json'   => 'JSON::Alexandria',
         'text/x-json'        => 'JSON::Alexandria',

         'text/html'          => 'YAML::HTML',
         'text/x-data-dumper' => [ 'Data::Serializer', 'Data::Dumper' ],
         'text/x-data-denter' => [ 'Data::Serializer', 'Data::Denter' ],
         'text/x-data-taxi'   => [ 'Data::Serializer', 'Data::Taxi'   ],
         'application/x-storable'   => [ 'Data::Serializer', 'Storable' ],
         'application/x-freezethaw' => [ 'Data::Serializer', 'FreezeThaw' ],
         'text/x-config-general'    => [ 'Data::Serializer', 'Config::General' ],
         'text/x-php-serialization' => [ 'Data::Serializer', 'PHP::Serialization' ],
        }
);


__PACKAGE__->meta->make_immutable;

1;
