package Alexandria::CDWS::Controller::Infrastructure;
use Moose;
use namespace::autoclean;

BEGIN {extends 'Catalyst::Controller::REST'; }


=head1 NAME

Alexandria::Controller::Infrastructure  Infrastructure controller


=head1 DESCRIPTION

Catalyst Controller.

=head1 METHODS

=cut

use Data::Dumper;
sub _post_params {
  my ( $self, $c ) = @_;
  my %params;

  my $data;
  if($c->req->content_type eq 'application/x-www-form-urlencoded') {
$c->log->debug( __PACKAGE__ . ':: content-type: ' . $c->request->content_type );
    $data = $c->request->parameters;
  }
  else {
    if(ref($c->request->body) =~ /File/) {
$c->log->debug( __PACKAGE__ . ':: FileRef' );
      my $tmp = $c->request->body;
      $tmp->seek(0,0);
      $data = do { local $/; <$tmp> };
    }
    elsif($c->request->body) {
$c->log->debug( __PACKAGE__ . ':: RequestBody' );
      $data = $c->request->body;
    }
    else {
$c->log->debug( __PACKAGE__ . ':: RequestParameters' );
      my @a = ();
      my $h = $c->request->parameters;
      foreach my $key ( keys %{$h} ) {
        if(ref($h->{$key}) =~ /ARRAY/) {
          foreach my $f ( @{$h->{$key}} ) {
            push(@a, "$key=$f");
          }
        }
        else { push(@a, "$key=$h->{$key}"); }
      }
      $data = join('&', @a);
    }
  }
  return $data;
}

sub execute : Local : ActionClass('REST') { }
sub execute_GET  :Args(0){
  my ( $self, $c ) = @_;

  my $ret = { dump => Dumper( $c->request ) };
  return $self->status_ok( $c, entity => $ret );
}

sub execute_POST  :Args(0){
  my ( $self, $c ) = @_;

  my $ret = $self->_post_params( $c );
  # $c->log->debug( Dumper( $c->request ) );
  # my $ret = { dump => Dumper( $c->request ) };
  my %resp = (
    content => $c->model('Zabbix')->execute( $ret )
  );
  return $self->status_ok( $c, entity => \%resp );
}

=head2 end

=cut

use XML::LibXML; # this xml is specific

sub serialize : ActionClass('Serialize') {}
sub end :Private {
  my ($self, $c) = @_;
  $c->forward('serialize');
}


=head1 AUTHOR

David Santamauro

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->config(
  'default'   => 'text/xml',
  'stash_key' => 'rest',
  'map'       => {

         'text/xml'           => 'XML::Alexandria::XResponse::XEcho',
         'application/json'   => 'JSON::Alexandria',
         'text/x-json'        => 'JSON::Alexandria',

         'text/html'          => 'YAML::HTML',
         'text/x-data-dumper' => [ 'Data::Serializer', 'Data::Dumper' ],
         'text/x-data-denter' => [ 'Data::Serializer', 'Data::Denter' ],
         'text/x-data-taxi'   => [ 'Data::Serializer', 'Data::Taxi'   ],
         'application/x-storable'   => [ 'Data::Serializer', 'Storable' ],
         'application/x-freezethaw' => [ 'Data::Serializer', 'FreezeThaw' ],
         'text/x-config-general'    => [ 'Data::Serializer', 'Config::General' ],
         'text/x-php-serialization' => [ 'Data::Serializer', 'PHP::Serialization' ],
        }
);


__PACKAGE__->meta->make_immutable;

1;
