package Alexandria::CDWS::Controller::PITR;
use Moose;
use namespace::autoclean;

use JSON ();
use XML::LibXML;
use Data::Dumper;
use URI::Escape;
use Time::HiRes qw( gettimeofday tv_interval );

BEGIN {extends 'Catalyst::Controller::REST'; }

=head1 NAME

Alexandria::Controller::PITR - main PITR interface


=head1 DESCRIPTION

PITR Catalyst Controller.

=head1 METHODS

=cut

sub _build_rankings {
  my ( $self, $c, $data ) = @_;
  my @cur_rank = ();
  my @cur_named_rank = ();

  my $cur_year = (localtime)[5] + 1900;
  my $pivot_year = $data->{facet_counts}->{facet_pivot}->{'pdyear,ifi_stdnamestd'};
  foreach my $piv ( @{ $pivot_year } ) {
    if( $piv->{value} == $cur_year ) {
      my $x = $piv->{pivot}; # array ref
      foreach my $s ( @{ $x } ) {
        push( @cur_rank, { name => $s->{value}, count => $s->{count} } );
      }
#$c->log->debug( Dumper ( \@cur_rank ) );
    }
  }
  my $pivot_name = $data->{facet_counts}->{facet_pivot}->{'ifi_stdnamestd,pdyear'};
  foreach my $piv ( @{ $pivot_name } ) { # order here is by count, not index
    my %name_struct;

    # { field, count, value, pivot }
    # field: ifi_stdnamestd
    # count: n
    # value: name
    # pivot: [ { value: year, count: n, field: pdyear } ]
 
    my $x = $piv->{pivot}; # array ref
    my %cy = ();

    my @years = sort( { $a->{value} <=> $b->{value} } @{ $x } );
    my @a = ();
    foreach my $s ( @years ) {
      push( @a, { year => $s->{value}, count => $s->{count} } );
    }
#$c->log->debug( Dumper ( \@a ) );
   $name_struct{ $piv->{value} }->{country} = 'US';
   $name_struct{ $piv->{value} }->{stats} = \@a;

    push( @cur_named_rank, \%name_struct );
  }

  my %s = (
    $cur_year => \@cur_rank,
    by_year => \@cur_named_rank
  );
  return \%s;
}

sub _handle_search {
  my ( $self, $c, $params ) = @_;

  my $t = [gettimeofday];

  my $user = $c->user;
  my ($content_type) = grep { $self->{map}->{$_} } @{$c->request->accepted_content_types};

  my ( $start_year, $end_year ) = (
    ( (localtime)[5] + 1900 ) - 5,
    ( (localtime)[5] + 1900 )
  );
  my $fq = ( $params->{u} ) ? "( pnkind:(A OR B*) ) +pdyear:[ $start_year TO $end_year ]" : "( pnkind:(A1 OR A2 OR A9) ) +pdyear:[ $start_year TO $end_year ]";
  my $rows = ( $params->{n} ) ? $params->{n} : 10;
  if( $rows > 1000 ) { $rows = 1000; }
  my %solropts = (
    fq => $fq,
    echoParams => 'none',
    start => 0,
    rows => 0,
    wt => 'json',
    indent => 'true',
    facet => 'true',
    'facet.field' => 'ifi_stdnamestd',
    'facet.limit' => $rows,
    'facet.mincount' => 1,
    'facet.pivot' => [ 'pdyear,ifi_stdnamestd', 'ifi_stdnamestd,pdyear' ],
    'facet.pivot.mincount' => 1,
    'facet.pivot.limit' => $rows
  );

  my $srchmodel = $c->model('Solr::PITR');
  my $rdbmodel = $c->model( $user->idx_model );

  # set timeout
  $srchmodel->agent->timeout(600);

$c->log->debug( Dumper (\%solropts ) );

  # save locally and remove q
  my $q = '*:*';

  ### finally, the search ...
  my $response = $srchmodel->search( $q, \%solropts );

  my $x = $self->_build_rankings( $c, $response->content );

  my %s = (
    status => 'success',
    time => sprintf("%.3f", tv_interval( $t ) ),
    content => $x
    #content => $response->content
  );
#$c->log->debug( Dumper ( \%s ) );
  return $self->status_ok( $c, entity => \%s );
}

sub topn : Local : ActionClass('REST') { }
sub topn_GET  :Args(0){
  my ( $self, $c ) = @_;
  return $self->_handle_search( $c, $c->request->parameters );
}

sub _post_params {
  my ( $self, $c ) = @_;
  my %params;

#$c->log->debug( Dumper( $c->request ) );
  if($c->req->content_type eq 'application/x-www-form-urlencoded') {
$c->log->debug( __PACKAGE__ . ':: content-type: ' . $c->request->content_type );
    my $s = $c->request->parameters;
    foreach my $key ( keys %{$s} ) {
      my $val = $s->{$key};
      $val =~ s/^\s+//;
      $val =~ s/\s+$//;
      $params{$key} = $val;
    }
  }
  else {
    my $data;
    if(ref($c->request->body) =~ /File/) {
$c->log->debug( __PACKAGE__ . ':: FileRef' );
      my $tmp = $c->request->body;
      $tmp->seek(0,0);
      $data = do { local $/; <$tmp> };
    }
    elsif($c->request->body) {
$c->log->debug( __PACKAGE__ . ':: RequestBody' );
      $data = $c->request->body;
    }
    else {
$c->log->debug( __PACKAGE__ . ':: RequestParameters' );
      my @a = ();
      my $h = $c->request->parameters;
      foreach my $key ( keys %{$h} ) {
        if(ref($h->{$key}) =~ /ARRAY/) {
          foreach my $f ( @{$h->{$key}} ) {
            push(@a, "$key=$f");
          }
        }
        else { push(@a, "$key=$h->{$key}"); }
      }
      $data = join('&', @a);
    }
#$c->log->debug( __PACKAGE__ . ":: DATA: $data" );
    my $str = uri_unescape($data);
    $str =~ s/^\s+//;
    $str =~ s/\s+$//;
    my @a = split(/\&/, $str);
    foreach my $e (@a) {
      my ($key, $val) = split(/=/, $e);
      $params{$key} = $val;
    }
  }
  return \%params;
}

sub topn_POST  :Args(0){
  my ( $self, $c ) = @_;
  return $self->_handle_search( $c, $self->_post_params( $c ) );
}

=head2 end

=cut

sub serialize : ActionClass('Serialize') {}
sub end :Private {
  my ($self, $c) = @_;
  # CAUTION: unknown side affects. Somehow, a POST
  # request, regardless of content-type and response->code,
  # sets the body to "Content-Type xxx is not supported"
  # For successul request/response, undef this because the
  # serializer sets the body
  if( $c->response->code == 200 && $c->response->has_body ) {
    $c->response->body(undef);
  }
  $c->forward('serialize');
}


=head1 AUTHOR

David Santamauro

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->config(
  'default'   => 'application/json',
  'stash_key' => 'rest',
  'map'       => {

         'text/xml'           => 'XML::Alexandria',
         'application/json'   => 'JSON::Alexandria',
         'text/x-json'        => 'JSON::Alexandria',

         'text/html'          => 'YAML::HTML',
         'text/x-data-dumper' => [ 'Data::Serializer', 'Data::Dumper' ],
         'text/x-data-denter' => [ 'Data::Serializer', 'Data::Denter' ],
         'text/x-data-taxi'   => [ 'Data::Serializer', 'Data::Taxi'   ],
         'application/x-storable'   => [ 'Data::Serializer', 'Storable' ],
         'application/x-freezethaw' => [ 'Data::Serializer', 'FreezeThaw' ],
         'text/x-config-general'    => [ 'Data::Serializer', 'Config::General' ],
         'text/x-php-serialization' => [ 'Data::Serializer', 'PHP::Serialization' ],
        }
);


__PACKAGE__->meta->make_immutable;

1;
