package Alexandria::CDWS::Controller::Remote;

use Moose;
use namespace::autoclean;
use Time::HiRes qw( gettimeofday tv_interval );
BEGIN {extends 'Catalyst::Controller::REST'; }

=head1 NAME

Alexandria::Controller::Remote - Catalyst Controller for remote updates

=head1 DESCRIPTION

Catalyst Controller.

=head1 METHODS

=cut

=head2 index

=cut
use Data::Dumper;

sub check : Local : ActionClass('REST') { }
sub check_GET  :Args(0){
  my ( $self, $c ) = @_;
  my $user = $c->user;

  my $t = [gettimeofday];

  my $params = $c->request->parameters;

  if( !$params->{max_client_modified_load_id} ) {
    return $self->status_bad_request( $c, message => "Missing required max_client_modified_load_id parameter" );
  }

  my $li;
  my @excluded_load_sources = ();
  eval {
    $li = $c->model( $user->rdb_model  )->loads_pending(
                $params->{max_client_modified_load_id},
                \@excluded_load_sources
    );
  };
  if( $@ ) {
  }

  my %ret = (
    status => 'success',
    time => sprintf( "%.6f", tv_interval($t) ),
    update => $li,
  );
  return $self->status_ok( $c, entity => \%ret );
}

sub update : Local : ActionClass('REST') { }
sub update_GET  :Args(0){
  my ( $self, $c ) = @_;
  my $user = $c->user;

  my $t = [gettimeofday];

  my $params = $c->request->parameters;

  if( !$params->{max_client_modified_load_id} ) {
    return $self->status_bad_request( $c, message => "Missing required max_client_modified_load_id parameter" );
  }

  my %ret = (
    status => 'success',
    time => sprintf( "%.6f", tv_interval($t) )
  );
  return $self->status_ok( $c, entity => \%ret );
}

=head2 end

=cut

sub serialize : ActionClass('Serialize') {}
sub end :Private {
  my ($self, $c) = @_;
  $c->forward('serialize');
}


=head1 AUTHOR

David Santamauro

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->config(
  'default'   => 'application/json',
  'stash_key' => 'rest',
  'map'       => {
         'text/html'          => 'YAML::HTML',
         'text/xml'           => 'XML::Simple',
         'application/json'   => 'JSON::Alexandria',
         'text/x-json'        => 'JSON::Alexandria',
         'text/x-data-dumper' => [ 'Data::Serializer', 'Data::Dumper' ],
         'text/x-data-denter' => [ 'Data::Serializer', 'Data::Denter' ],
         'text/x-data-taxi'   => [ 'Data::Serializer', 'Data::Taxi'   ],
         'application/x-storable'   => [ 'Data::Serializer', 'Storable' ],
         'application/x-freezethaw' => [ 'Data::Serializer', 'FreezeThaw' ],
         'text/x-config-general'    => [ 'Data::Serializer', 'Config::General' ],
         'text/x-php-serialization' => [ 'Data::Serializer', 'PHP::Serialization' ],
        }
);


__PACKAGE__->meta->make_immutable;

1;
