package Alexandria::CDWS::Controller::Services;
use Moose;
use namespace::autoclean;

BEGIN {extends 'Catalyst::Controller::REST'; }


=head1 NAME

Alexandria::Controller::Services - List available services


=head1 DESCRIPTION

Catalyst Controller.

=head1 METHODS

=cut

use Data::Dumper;

sub list : Local : ActionClass('REST') { }
sub list_GET  :Args(0){
  my ( $self, $c ) = @_;

  my @a = ();
  foreach my $ctrl ( sort $c->controllers ) {
    my $comp = $c->controller( $ctrl );
    push(@a, { name => $ctrl, namespace => $comp->action_namespace($c) } );
  }
  return $self->status_ok( $c, entity => \@a );
}

=head2 end

=cut

use XML::LibXML; # this xml is specific

sub serialize : ActionClass('Serialize') {}
sub end :Private {
  my ($self, $c) = @_;
  $c->forward('serialize');
}


=head1 AUTHOR

David Santamauro

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->config(
  'default'   => 'application/json',
  'stash_key' => 'rest',
  'map'       => {

         'text/xml'           => 'XML::Simple',
         'application/json'   => 'JSON::Alexandria::JSONResponse',
         'text/x-json'        => 'JSON::Alexandria::JSONResponse',

         'text/html'          => 'YAML::HTML',
         'text/x-data-dumper' => [ 'Data::Serializer', 'Data::Dumper' ],
         'text/x-data-denter' => [ 'Data::Serializer', 'Data::Denter' ],
         'text/x-data-taxi'   => [ 'Data::Serializer', 'Data::Taxi'   ],
         'application/x-storable'   => [ 'Data::Serializer', 'Storable' ],
         'application/x-freezethaw' => [ 'Data::Serializer', 'FreezeThaw' ],
         'text/x-config-general'    => [ 'Data::Serializer', 'Config::General' ],
         'text/x-php-serialization' => [ 'Data::Serializer', 'PHP::Serialization' ],
        }
);


__PACKAGE__->meta->make_immutable;

1;
