package Alexandria::CDWS::Controller::System::DWH::History;
use Moose;
use namespace::autoclean;
use Time::HiRes qw( gettimeofday tv_interval );
BEGIN {extends 'Catalyst::Controller::REST'; }


=head1 NAME

Alexandria::Controller::System::DWH::History - Catalyst Controller

All things loading per ucid

=head1 DESCRIPTION

Catalyst Controller.

=head1 METHODS

=cut

=head2 index

=cut
use Data::Dumper;

sub info : Local : ActionClass('REST') { }
sub info_GET  :Args(0){
  my ( $self, $c ) = @_;
  my $user = $c->user;

  my $t = [gettimeofday];

  my $params = $c->request->parameters;

  if( !$params->{ucid} ) {
    return $self->status_bad_request( $c, message => "Missing required ucid parameter" );
  }
  my $history = $c->model( 'MSSQL' )->get_system_history_by_ucid( $params->{ucid} );

  my %ret = (
    status => 'success',
    time => sprintf( "%.6f", tv_interval($t) ),
    history => $history
  );
#$c->log->debug(Dumper(\%ret));
  return $self->status_ok( $c, entity => \%ret );
}

=head2 end

=cut

use XML::LibXML; # this xml is specific

sub serialize : ActionClass('Serialize') {}
sub end :Private {
  my ($self, $c) = @_;
  $c->forward('serialize');
}


=head1 AUTHOR

David Santamauro

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->config(
  'default'   => 'application/json',
  'stash_key' => 'rest',
  'map'       => {
         'text/html'          => 'YAML::HTML',
         'text/xml'           => 'XML::Alexandria::XResponse::XSystem',
         'application/json'   => 'JSON::Alexandria',
         'text/x-json'        => 'JSON::Alexandria',
         'text/x-data-dumper' => [ 'Data::Serializer', 'Data::Dumper' ],
         'text/x-data-denter' => [ 'Data::Serializer', 'Data::Denter' ],
         'text/x-data-taxi'   => [ 'Data::Serializer', 'Data::Taxi'   ],
         'application/x-storable'   => [ 'Data::Serializer', 'Storable' ],
         'application/x-freezethaw' => [ 'Data::Serializer', 'FreezeThaw' ],
         'text/x-config-general'    => [ 'Data::Serializer', 'Config::General' ],
         'text/x-php-serialization' => [ 'Data::Serializer', 'PHP::Serialization' ],
        }
);


__PACKAGE__->meta->make_immutable;

1;
