package Alexandria::CDWS::Controller::System::DWH::Loading;
use Moose;
use namespace::autoclean;
use Time::HiRes qw( gettimeofday tv_interval );
BEGIN {extends 'Catalyst::Controller::REST'; }


=head1 NAME

Alexandria::Controller::System::DWH::Loading - Catalyst Controller

All things postgresql loading

=head1 DESCRIPTION

Catalyst Controller.

=head1 METHODS

=cut

=head2 index

=cut
use Data::Dumper;

sub info : Local : ActionClass('REST') { }
sub info_GET  :Args(0){
  my ( $self, $c ) = @_;
  my $user = $c->user;

  my $t = [gettimeofday];

  my $params = $c->request->parameters;

  if( !$params->{load_id} ) {
    return $self->status_bad_request( $c, message => "Missing required load_id parameter" );
  }
  my $info = $c->model( $user->rdb_model )->loadid_information( $params->{load_id} );

  if( !$info->{load}->{load_source} ) {
    return $self->status_no_content( $c, message => "No information available for $params->{load_id}" );
  }
#$c->log->debug(Dumper($info));
  $info->{status} = 'success';
  $info->{time} = sprintf( "%.6f", tv_interval($t) );
  return $self->status_ok( $c, entity => $info );
}

=head2 end

=cut

use XML::LibXML; # this xml is specific

sub serialize : ActionClass('Serialize') {}
sub end :Private {
  my ($self, $c) = @_;
  $c->forward('serialize');
}


=head1 AUTHOR

David Santamauro

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->config(
  'default'   => 'application/json',
  'stash_key' => 'rest',
  'map'       => {
         'text/html'          => 'YAML::HTML',
         'text/xml'           => 'XML::Alexandria::XResponse::XSystem',
         'application/json'   => 'JSON::Alexandria',
         'text/x-json'        => 'JSON::Alexandria',
         'text/x-data-dumper' => [ 'Data::Serializer', 'Data::Dumper' ],
         'text/x-data-denter' => [ 'Data::Serializer', 'Data::Denter' ],
         'text/x-data-taxi'   => [ 'Data::Serializer', 'Data::Taxi'   ],
         'application/x-storable'   => [ 'Data::Serializer', 'Storable' ],
         'application/x-freezethaw' => [ 'Data::Serializer', 'FreezeThaw' ],
         'text/x-config-general'    => [ 'Data::Serializer', 'Config::General' ],
         'text/x-php-serialization' => [ 'Data::Serializer', 'PHP::Serialization' ],
        }
);


__PACKAGE__->meta->make_immutable;

1;
