package Alexandria::CDWS::Controller::Update::Output;
use Moose;
use namespace::autoclean;

BEGIN {extends 'Catalyst::Controller::REST'; }


=head1 NAME

Alexandria::Controller::Update::Output - Remote update controller accepting client load files

=head1 DESCRIPTION

Catalyst Controller.

=head1 METHODS

=cut

use Data::Dumper;

# sub deserialize : ActionClass('Deserialize') {}
sub deserialize  :ActionClass('DeserializeMultiPart') {}
sub begin : Private {
  my ($self, $c) = @_;
  # $c->forward('deserialize');
}

sub _s {
  my ( $self, $c ) = @_;

  my $params = $c->request->parameters;
  my $upload = $c->request->upload('file');

$c->log->debug( Dumper( $upload ) );

  my $storage = $c->model('Update')->{storage};
  if( !$storage || !( -W $storage ) ) {
    my $ret = { status => 'error', message => 'failed to find suitable storage' };
    $c->log->error("failed to find suitable storage: $storage");
    return $self->status_ok( $c, entity => $ret );
  }

  my $dest = "$storage/$params->{id}";
  if( !(-d "$dest") ) {
    mkdir "$dest" ||
      return $self->status_ok( $c, entity => { status => 'error', message => $! } );
  }
  $upload->copy_to( sprintf("%s/%s", $dest, $upload->filename ) ) ||
      return $self->status_ok( $c, entity => { status => 'error', message => 'failed to copy file to storage' } );

  my $ret = { status => 'success' };
  return $self->status_ok( $c, entity => $ret );
}

sub save : Local : ActionClass('REST') { }
sub save_PUT  :Args(0){
  my $self = shift;
  return $self->_s(@_);
}

sub save_POST  :Args(0){
  my $self = shift;
  return $self->_s(@_);
}

=head2 end

=cut

sub serialize : ActionClass('Serialize') {}
sub end :Private {
  my ($self, $c) = @_;
  $c->forward('serialize');
}


=head1 AUTHOR

David Santamauro

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->config(
  'default'   => 'application/json',
  'stash_key' => 'rest',
  'map'       => {

         'text/xml'           => 'XML::Simple',
         'application/json'   => 'JSON',
         'text/x-json'        => 'JSON',

         'text/html'          => 'YAML::HTML',
         'text/x-data-dumper' => [ 'Data::Serializer', 'Data::Dumper' ],
         'text/x-data-denter' => [ 'Data::Serializer', 'Data::Denter' ],
         'text/x-data-taxi'   => [ 'Data::Serializer', 'Data::Taxi'   ],
         'application/x-storable'   => [ 'Data::Serializer', 'Storable' ],
         'application/x-freezethaw' => [ 'Data::Serializer', 'FreezeThaw' ],
         'text/x-config-general'    => [ 'Data::Serializer', 'Config::General' ],
         'text/x-php-serialization' => [ 'Data::Serializer', 'PHP::Serialization' ],
        }
);


__PACKAGE__->meta->make_immutable;

1;
