package Alexandria::CDWS::Controller::Version;
use Moose;
use namespace::autoclean;
use Time::HiRes qw( gettimeofday tv_interval );
BEGIN {extends 'Catalyst::Controller::REST'; }


=head1 NAME

Alexandria::Controller::Version


=head1 DESCRIPTION

Catalyst Controller.

=head1 METHODS

=cut

use Data::Dumper;

sub show : Local : ActionClass('REST') { }
sub show_GET  :Args(0){
  my ( $self, $c ) = @_;

  my $t = [gettimeofday];

  my $params = $c->request->parameters;

  my $v = $c->config->{version};
  my ( $maj, $min ) = split(/\./, $v );
  my %ret = (
    time => sprintf( '%.3f', tv_interval( $t ) ),
    status => 'success',
    version => { string => "adw-cdws-v" . $v, major => $maj, minor => $min }
  );

  return $self->status_ok( $c, entity => \%ret );
}

=head2 end

=cut

use XML::LibXML; # this xml is specific

sub serialize : ActionClass('Serialize') {}
sub end :Private {
  my ($self, $c) = @_;
  $c->forward('serialize');
}


=head1 AUTHOR

David Santamauro

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->config(
  'default'   => 'application/json',
  'stash_key' => 'rest',
  'map'       => {

         'text/xml'           => 'XML::Alexandria',
         'application/json'   => 'JSON::Alexandria',
         'text/x-json'        => 'JSON::Alexandria',

         'text/html'          => 'YAML::HTML',
         'text/x-data-dumper' => [ 'Data::Serializer', 'Data::Dumper' ],
         'text/x-data-denter' => [ 'Data::Serializer', 'Data::Denter' ],
         'text/x-data-taxi'   => [ 'Data::Serializer', 'Data::Taxi'   ],
         'application/x-storable'   => [ 'Data::Serializer', 'Storable' ],
         'application/x-freezethaw' => [ 'Data::Serializer', 'FreezeThaw' ],
         'text/x-config-general'    => [ 'Data::Serializer', 'Config::General' ],
         'text/x-php-serialization' => [ 'Data::Serializer', 'PHP::Serialization' ],
        }
);


__PACKAGE__->meta->make_immutable;

1;
