package Alexandria::CDWS::Model::DWH::Standard;

use strict;
use Data::Dumper;
use base 'Alexandria::CDWS::Model::DWH';

use utf8;

use XML::LibXML;
use JSON;
use Time::HiRes qw(gettimeofday tv_interval);
################ FWS fragment
sub fws_members {
  my ($self, $fam, $ctry, $ucid, $anucid, $origin, $ftflag ) = @_;
  my $c = $self->{c};
  my @family = ();
  my $sth = $self->dbh->prepare( "select * from fws.f_members( ?,?,?,?,?,? )" );
  $sth->execute( $fam, $ctry, $ucid, $anucid, ( $origin ) ? $origin : $ctry, $ftflag );
  while( my $ref = $sth->fetchrow_hashref() ) {
    my %s = ();
    foreach my $key ( keys %{ $ref } ) { $s{$key} = $ref->{$key}; }
    push( @family, \%s );
  }
  return \@family;
}

sub fws_ls_effective_grant_date {
  my ( $self, $ucid ) = @_;
  my $c = $self->{c};
  my $date;

  eval {
    my $sth = $self->dbh->prepare(
                 "select date from ctws.f_ls_trace(?)
                  where country='JP' and code='R150'
                  order by date limit 1"

    );
    $sth->execute( $ucid );
    ( $date ) = $sth->fetchrow_array();
    $sth->finish();
  };
  if($@) { $c->log->error( $@ ); }
  return $date;
}

sub fws_ls_corresponding_documents {
  my ( $self, $ucid ) = @_;
  my $c = $self->{c};
  my $event;

  eval {
    my $sth = $self->dbh->prepare(
                 "select date from ctws.f_ls_trace(?)
                  where country='EP' and code='REF'
                  order by date limit 1"

    );
    $sth->execute( $ucid );
    ( $event ) = $sth->fetchrow_hashref();
    $sth->finish();
  };
  if($@) { $c->log->error( $@ ); }
$c->log->debug( Dumper( $event ) );
  return $event;
}

sub fws_ls_national_grants_information {
  my ( $self, $ucid ) = @_;
  my $c = $self->{c};
  my @ret = ();

  eval {
    my $sth = $self->dbh->prepare( "select * from fws.f_ls_national_grants_information(?)" );
    $sth->execute( $ucid );
    while( my $ref = $sth->fetchrow_hashref() ) {
      push( @ret, { ctry => $ref->{ctry}, dnum => $ref->{dnum}, date => $ref->{date} } );
    }
    $sth->finish();
  };
  if($@) { $c->log->error( $@ ); }
#$c->log->debug( Dumper( \@ret ) );
  return \@ret;
}

################ TWS JSON fragment
sub acws_container {
  my ($self, $ucids, $full ) = @_;
  my $c = $self->{c};

  my %s = ();
  eval {
    my $sth = $self->dbh->prepare( "select * from acws.f_tws_container( ?, ? )");
    $sth->execute( $ucids, $full );

    $s{xml} = '<patent-documents>';
    while( my $ref = $sth->fetchrow_arrayref() ) {
      push(@{ $s{publication_id} }, $ref->[0] );
      push(@{ $s{ucid} },           $ref->[1] );
      push(@{ $s{kind} },           $ref->[2] );
      push(@{ $s{family_id} },      $ref->[3] );
      $s{xml} .= $ref->[4];
    }
    $s{xml} .= '</patent-documents>';
    $sth->finish();
  };
#$c->log->debug( Dumper( \%s ) );
  return \%s;
}

sub tws_container {
  my ( $self, $ucids, $full ) = @_;
  my $c = $self->{c};

  my $p = XML::LibXML->new( huge => 1 );
  my $json_str;
  eval {
    my $sth = $self->dbh->prepare( "select ctws.f_tws_container( ?, ? )");
    $sth->execute( $ucids, $full );
    while( my $ref = $sth->fetchrow_arrayref() ) {
      $json_str = $ref->[0];
    }
    $sth->finish();
  };
  if($@) { $c->log->error( $@ ); }

  if( $json_str ) {
    my $obj;
    my $json = JSON->new->utf8->convert_blessed;
      $json->pretty(1);
    return $json->decode( $json_str );
  }
  return undef;
}

sub tws_ls_container {
  my ( $self, $ucids ) = @_;
  my $c = $self->{c};

  my @ucid_strs;
  foreach my $ucid ( @{ $ucids} ) {
    push(@ucid_strs, "\'$ucid\'");
  }

  my $p = XML::LibXML->new( huge => 1 );
  my $root = XML::LibXML::Element->new('patent-documents');
  eval {
    my $sql = sprintf(
        "select ctws.f_patent_document_ls(t.publication_id, null )
         from xml.t_patent_document_values as t
         where ucid in (%s)", join(',', @ucid_strs)
    );
    my $sth = $self->dbh->prepare( $sql );
    $sth->execute( );
    while( my $ref = $sth->fetchrow_arrayref() ) {
      my $x = $p->parse_string( $ref->[0] );
      $root->addChild( $x->documentElement() );
    }
    $sth->finish();
  };
  if($@) { $c->log->error( $@ ); }

  return $root;
}

sub tws_ls_corresponding_documents {
  my ( $self, $ucid ) = @_;
  my $c = $self->{c};
  my @ret = ();

  eval {
    my $sth = $self->dbh->prepare( "select * from ctws.f_corresponding_documents(?)" );
    $sth->execute( $ucid );
    while( my $ref = $sth->fetchrow_arrayref() ) {
      push( @ret, $ref->[0] );
    }
    $sth->finish();
  };
  if($@) { $c->log->error( $@ ); }
#$c->log->debug( Dumper( \@ret ) );
  return \@ret;
}

sub tws_ls_national_grants_information {
  my ( $self, $ucid ) = @_;
  my $c = $self->{c};
  my @ret = ();

  eval {
    my $sth = $self->dbh->prepare( "select * from ctws.f_national_grants_information(?)" );
    $sth->execute( $ucid );
    while( my $ref = $sth->fetchrow_hashref() ) {
      push( @ret, { ctry => $ref->{ctry}, dnum => $ref->{dnum}, date => $ref->{date} } );
    }
    $sth->finish();
  };
  if($@) { $c->log->error( $@ ); }
#$c->log->debug( Dumper( \@ret ) );
  return \@ret;
}

sub tws_ls_effective_grant_date {
  my ( $self, $ucid ) = @_;
  my $c = $self->{c};
  my $date;

  eval {
    my $sth = $self->dbh->prepare(
                 "select date from ctws.f_ls_trace(?)
                  where country='JP' and code='R150'
                  order by date limit 1"

    );
    $sth->execute( $ucid );
    ( $date ) = $sth->fetchrow_array();
    $sth->finish();
  };
  if($@) { $c->log->error( $@ ); }
  return $date;
}

sub tws_ls_corresponding_documents_ex {
  my ( $self, $ucids ) = @_;
  my $c = $self->{c};
  my @ret = ();

  eval {
    my $sth = $self->dbh->prepare( "select * from ctws.f_corresponding_documents_ex(?)" );
    $sth->execute( $ucids );
    while( my $ref = $sth->fetchrow_arrayref() ) {
      push( @ret, $ref->[0] );
    }
    $sth->finish();
  };
  if($@) { $c->log->error( $@ ); }

  return \@ret;
}


sub tws_document {
  my ( $self, $ucids ) = @_;
  my $c = $self->{c};

  my $ret;
  eval {
    my $sth = $self->dbh->prepare( "select ctws.f_tws_document( ? )");
    $sth->execute( $ucids );
    while( my $ref = $sth->fetchrow_arrayref() ) {
      $ret .= $ref->[0];
    }
    $sth->finish();
  };
  if($@) { $c->log->error( $@ ); }

  if($ret) {
    my $obj;
     my $json = JSON->new->utf8->convert_blessed;
    $json->pretty(1);
    return $json->decode( $ret );
  }
  return undef;
}

################ Raw XML Table Data
sub table_data_as_doc {
  my ( $self, $ucid, $table ) = @_;
  my $c = $self->{c};

  my $p = XML::LibXML->new( huge => 1 );
  my @content = ();
  eval {
    my $sth = $self->dbh->prepare(
        "select content
         from $table
         where publication_id=xml.f_ucid2id(?)");
    $sth->execute( $ucid );
    while( my $ref = $sth->fetchrow_arrayref() ) {
      push( @content, $p->parse_string( $ref->[0] ) );
    }
    $sth->finish();
  };
  if($@) { $c->log->error( $@ ); }
  return \@content;
}

################ CITATIONS
sub _extended_data {
  my ( $self, $cites ) = @_;
  my $c = $self->{c};
  my %extended_citations;

  foreach my $refucid ( @{ $cites } ) {
    # published | first inventor | first applicant first cpc
    my %s = ();
    my $sth = $self->dbh->prepare(
      "select to_char(a.published, 'YYYYMMDD') as published,
              reports.f_csv_name( xml.f_findNode( '//applicant', b.content ) ) as applicant,
              reports.f_csv_name( xml.f_findNode( '//inventor', b.content ) ) as inventor,
              reports.f_csv_name( xml.f_findNode( '//assignee', b.content ) ) as assignee,
              xml.f_textContent('//classification-cpc', ( xml.f_getElementsByTagName( 'classification-cpc', c.content ) )[1] ) as cpc
       from xml.t_patent_document_values a
         left outer join xml.t_parties b on ( a.publication_id=b.publication_id )
         left outer join xml.t_classifications_cpc c on ( a.publication_id=c.publication_id )
       where a.publication_id=xml.f_ucid2id( ? )"
    );
    $sth->execute( $refucid );

    while( my $data = $sth->fetchrow_arrayref() ) {
      $s{published} = $data->[0] if $data->[0];
      $s{applicant} = $data->[1] if $data->[1];
      $s{inventor}  = $data->[2] if $data->[2];
      $s{assignee}  = $data->[3] if $data->[3];
      $s{cpc}       = $data->[4] if $data->[4];
      $extended_citations{ $refucid } = \%s;
    }
    $sth->finish();
  }
  return \%extended_citations;
}

sub citations_backward {
  my ( $self, $ucid, $extdata ) = @_;
  my $c = $self->{c};

#$c->log->debug( "Fetching citation data ( $ucid )" );

  my @citations = ();
  my $extended_citations;
  eval {
    my $sth = $self->dbh->prepare(
      "select unnest( xpath(\'//patcit/\@ucid\', content) )
       from xml.t_citations
       where publication_id=xml.f_ucid2id(?)");
    $sth->execute( $ucid );
    while( my $data = $sth->fetchrow_arrayref() ) {
      push( @citations, $data->[0] );
    }
    $sth->finish();

    # extended data
    if( $extdata ) {
      $extended_citations = $self->_extended_data( \@citations );
    }
  };
  if($@) { $c->log->error( $@ ); }
  if( scalar( keys ( %{ $extended_citations } ) ) ) {
    return { root => $ucid, ucids => $extended_citations };
  }
  return { root => $ucid, ucids => \@citations };
}

################ FAMILY
sub family_simple {
  my ($self, $ucid) = @_;
  my $c = $self->{c};
  my @members = ();
  my $fam = -1;

  my $t = [gettimeofday];

  eval {
    my $sth = $self->dbh->prepare(
      "select ucid, family_id
       from xml.t_patent_document_values
       where family_id in (
         select b.family_id
         from xml.t_patent_document_values as b
         where b.publication_id=xml.f_ucid2id(?)
           and b.family_id != -1
       ) and ucid != ?
       order by ucid asc"
    );
    $sth->execute( $ucid, $ucid );
    while( my $data = $sth->fetchrow_arrayref() ) {
      push( @members, $data->[0] );
      $fam = $data->[1];
    }
  };
  if($@) { $c->log->error( $@ ); }

  my @a = sort(@members);
  return {
    _dwh_time => sprintf("%.6f", tv_interval( $t ) ),
    root => $ucid,
    family_id => $fam,
    members => \@a
  };
}

sub _family_extended_mssql {
  my ( $self, $ucid, $mssql ) = @_;

  my $c = $self->{c};
$c->log->debug( "family/extended: mssql only ( $ucid )" );
  my $t = [gettimeofday];

  my @a = ();
  my $mssql_family = $mssql->extended_inpadoc_family( $ucid );
  foreach my $e ( sort @{ $mssql_family } ) {
    push(@a, $e);
  }

  my %s = (
    root => $ucid,
    n_members => scalar(@a),
    members => \@a,
    db_time => sprintf("%.6f", tv_interval( $t ) ),
  );
  return \%s;

}

sub family_extended {
  my ($self, $ucid, $solr, $incbasic, $mssql, $only_mssql) = @_;

  if( $mssql && $only_mssql ) {
    return $self->_family_extended_mssql( $ucid, $mssql );
  }

  my ($pg_time, $mssql_time);

  my $t = [gettimeofday];

  my %members = ();

  my $c = $self->{c};

  my $p = XML::LibXML->new();

  # get simple family
  my $fam = $self->family_simple( $ucid );

  # STEP #2 get priority data from input publication_id
  my @m = @{$fam->{members} };
  # push root ... we want it's priorities as well
  push(@m, $ucid);

  my %priorities = ();

  # THE BASE FAMILY
  my $sth = $self->dbh->prepare("select content from xml.t_priority_claims where publication_id=xml.f_ucid2id(?)");
  foreach my $id ( @m ) {

    if($incbasic) { $members{$id} = 1; }

    $sth->execute( $id );
    while( my $ref = $sth->fetchrow_arrayref() ) {
      my $dom = $p->parse_string( $ref->[0] );
      my $root = $dom->documentElement();
      foreach my $node ( $root->getElementsByTagName('priority-claim') ) {
        if( $node->getAttribute('ucid') ) {
          my $key = sprintf( "\"%s\"", $node->getAttribute('ucid') );
          $priorities{ $key } = 1;
        }
      }
    }
  }
  $sth->finish();

#  $c->log->debug( Dumper(\%priorities ) );

  # STEP #2a search the above priorities in SOLR (pri)
  my %family_ids = ();

  # include original
  $family_ids{ $fam->{family_id} } = 1;

  my $q = sprintf("pri: %s", join( " OR ", keys(%priorities) ) );
#$c->log->debug( "PRIORITY: $q" );
  my %solrparams = ( qt => 'standard', fl => 'fam', rows => 1000 );
  my $response = $solr->search( $q, \%solrparams );
  my $content = $response->content;
  foreach my $doc ( @{ $content->{response}->{docs} } ) {
    $family_ids{ $doc->{fam} } = 1
      unless $doc->{fam} == -1; # don't include -1
  }

#  $c->log->debug( Dumper(\%family_ids) );

  # STEP #3 take all family_ids and grab all members ...
  $q = sprintf("( fam:(%s) AND (adyear:[1700 TO *]) )", join(" OR ", keys(%family_ids) ) );
  $solrparams{fl} = 'ucid';
  $response = $solr->search( $q, \%solrparams );
  $content = $response->content;
  foreach my $doc ( @{ $content->{response}->{docs} } ) {
    if( !exists( $members{ $doc->{ucid} } ) ) {
      $members{ $doc->{ucid} } = 1;
    }
  }
#  $c->log->debug( Dumper(\%members) );

  $pg_time = sprintf("%.6f", tv_interval( $t ) );

  my @a = sort( keys(%members) );
  my @b = ();
  my @c = ();

  $t = [gettimeofday];

  if($mssql) {
    my $mssql_family = $mssql->extended_inpadoc_family( $ucid );
    foreach my $e ( sort @{ $mssql_family } ) {
      push(@b, $e);
    }

    # if we are here, a comparison is required so we create a 3rd
    # structure of all uniq ucids marking the source of each as
    #   dwh : 0
    #   mssql: 1
    #   both: 2
    # from one or the other
    my %u = ();
    foreach my $ucid ( @a ) { $u{$ucid} = 0; }
    foreach my $ucid ( @b ) {
      if( $u{$ucid} == 0 ) { $u{$ucid} = 2; }
      else { $u{$ucid} = 1; }
    }
    # now the opposite
    foreach my $ucid ( @b ) { $u{$ucid} = 1; }
    foreach my $ucid ( @a ) {
      if( $u{$ucid} == 1 ) { $u{$ucid} = 2; }
      else { $u{$ucid} = 0; }
    }
    # push them ...
    foreach my $ucid ( sort keys %u ) { push(@c, { ucid => $ucid, source => $u{$ucid} } ); }
  }
  $mssql_time = sprintf("%.6f", tv_interval( $t ) );

  my %s = (
    root => $ucid,
    n_members => scalar(@a),
    members => \@a,
    dwh_time => $pg_time
  );
  if($mssql) {
    $s{n_dwh_members} = scalar(@a);
    $s{mssql_time} = $mssql_time;
    $s{mssql_members} = \@b;
    $s{members_compared} = \@c;
    $s{n_mssql_members} = scalar(@b);
  }
  return \%s;
}

sub family_meta {
  my ($self, $ucid, $extended ) = @_;
  my $c = $self->{c};
  my @members = ();

  my $t = [gettimeofday];

  eval {
    my $sth = $self->dbh->prepare( "select * from fws.f_family_meta( ? ) " );
    $sth->execute( $ucid );
    while( my $ref = $sth->fetchrow_hashref() ) {
      push( @members,  $ref );
    }
  };
  if($@) { $c->log->error( $@ ); }

  return {
    time => sprintf("%.6f", tv_interval( $t ) ),
    members => \@members
  };
}


################ TEXT
sub _generate_text_container_sth {
  my $self = shift;
  my $ucids = shift;
  my $containers = shift;

  # read-only
  my $c = $self->{c};
  my $user = $c->user;

#$c->log->debug( Dumper($user) );

  # Pay attention to: (t_)ifi_integrated_content
  my $sth;

  my @tables = ( );

  my $idx = 1;
  foreach my $cont ( @{ $containers } ) {
    push( @tables, sprintf( "inner join xml.t_%s as t%d on (t%d.publication_id=t%d.publication_id)", $cont, $idx, $idx, $idx - 1 ) );
    $idx++;
  }
 
  my @sql = ( 'select t0.*,' );
  my @fields = ();
  for( my $i = 0; $i < scalar(@tables); $i++ ) {
    push( @fields, sprintf( "t%d.content", $i ) );
  }
  push( @sql, sprintf( "%s", join(',', @fields ) ) );
  push( @sql, "from t_patent_document_values as t0" );
  push( @sql, sprintf( "%s", join(' ', @tables ) ) );
  push( @sql, "where t0.publication_id=xml.f_ucid2id(?)" );
$c->log->debug( join( ' ', @sql ) );
}

sub _generate_text_sth {
  my $self = shift;
  my $ucids = shift;
  my $containers = shift;
  # read-only
  my $c = $self->{c};
  my $user = $c->user;

#  $self->_generate_text_container_sth( $ucids, $containers );

#$c->log->debug( Dumper($user) );

  my $sth;

  my %ftbl = ( single => 'xml.f_patent_document_no_ifi', multi => 'xml.f_patent_document_no_ifi_s' );
  if( $user->has_privilege('Platinum') ) {
    $ftbl{single} = 'xml.f_patent_document';
    $ftbl{multi} = 'xml.f_patent_document_s';
  }
  if(scalar(@{$ucids}) == 1) {
    $sth = $self->dbh->prepare("select $ftbl{single}(xml.f_ucid2id('$ucids->[0]'), null)");
  }
  else {
    my $tmpsth = $self->dbh->prepare("select 't_' || replace( cast( uuid_generate_v4() as text ), '-', '_')");
    $tmpsth->execute();
    my ($tblname) = $tmpsth->fetchrow_array();
    $tmpsth->finish();

    $self->dbh->do("create temp table if not exists $tblname (publication_id integer)");
    $self->dbh->do("truncate table $tblname");
    my $sth1 = $self->dbh->prepare("insert into $tblname (publication_id) values (xml.f_ucid2id(?))");
    foreach my $ucid (@{$ucids}) {
      $sth1->execute($ucid);
    }
    $sth1->finish();
    $sth = $self->dbh->prepare("select $ftbl{multi}(t.publication_id, null) from $tblname as t");
  }
  return $sth;
}

sub fetch_xml {
  my ($self, $c, $ucids, $cb, $containers) = @_;
  my $dbh = $self->dbh;

  my $sth = $self->_generate_text_sth( $ucids, $containers );
  $sth->execute();

  my $ret;

  while(my $ref = $sth->fetchrow_arrayref()) {
    if($cb) { &{$cb}($c, $ref->[0]); }
    else { $ret = $ref->[0]; }
  }
  return $ret;
}

################ SYSTEM
sub _cluster_stats {
  my $self = shift;
  my $dbh = $self->dbh;
  my %ret;

  # general statistics for the cluster
  my %queries = (
    version => "select version() as version",
    processes => "select sum(numbackends) as num_backends from pg_stat_database",
    pg_stat_bg_writer => "select * from pg_stat_bgwriter"
  );

  foreach my $q ( keys %queries ) {
    my $sth = $dbh->prepare( $queries{$q} );
    $sth->execute();
    my $ref = $sth->fetchrow_hashref();
    foreach my $k (keys %{$ref}) { $ret{$k} = $ref->{$k}; }
  }
  return \%ret;
}

sub _database_stats {
  my $self = shift;
  my $dbh = $self->dbh;

  my $dbname = $dbh->{Name};
  $dbname =~ s/^dbname=([^;].*);host.*$/$1/;

  # one row
  my $sth = $dbh->prepare( 'select * from pg_stat_database where datname = ? limit 1');
  $sth->execute( $dbname );

  my %ret = ();
  while( my $ref = $sth->fetchrow_hashref() ) {
    foreach my $k ( keys %{$ref} ) { $ret{$k} = $ref->{$k}; }
  }
  $sth->finish();
  return \%ret;
}

sub _load_stats_int {
  my $self = shift;
  my $tbl = shift;
  my $args = shift;
  my $nweeks = ($args && $args->{nweeks}) ? $args->{nweeks} : 1;
  my $sweeks = sprintf("'%d week%s'", $nweeks, ($nweeks > 1) ? 's' : '');
  my @ret = ();
  
  my $c = $self->{c};
  my $dbh = $self->dbh;
  my $sth;
  my $sql = "select load_id, load_source, ndocs, completed_status,
                   to_char(entered_stamp, 'YYYY-MM-DD HH24:MM:SS') as entered_stamp,
                   to_char(completed_stamp, 'YYYY-MM-DD HH24:MM:SS') as completed_stamp,
                   to_char(entered_stamp, 'DD-MM-YYYY') as entered_stamp_js,
                   to_char(completed_stamp, 'DD-MM-YYYY') as completed_stamp_js
             from reporting.$tbl
             where completed_stamp >= now() - interval $sweeks
               and entered_stamp is not null
             order by completed_stamp desc";
  if( $args && $args->{limit} ) {
    $sql .= " limit $args->{limit}";
  }
# $c->log->debug( $sql );
  $sth = $dbh->prepare( $sql );
  $sth->execute();

  while( my $ref = $sth->fetchrow_hashref() ) {
    my %s = ();
    foreach my $k ( keys %{$ref} ) {
      $s{$k} = $ref->{$k};
    }
    push(@ret, \%s);
  }
  return \@ret;
}

sub _index_stats {
  my $self = shift;
  my $args = shift;
  my $c = $self->{c};
  my $tbl = ($c->config->{remote}) ? 't_client_index_process' : 't_index_process';
  return $self->_load_stats_int($tbl, $args);
}

sub _load_stats {
  my $self = shift;
  my $args = shift;
  my $c = $self->{c};
  my $tbl = ($c->config->{remote}) ? 't_client_load_process' : 't_load_process';
  return $self->_load_stats_int($tbl, $args);
}

sub dwh_stats {
  my $self = shift;
  my $params = shift;

  if($params && $params->{component} && $params->{component} eq 'queue') {
    # ARRAYREF
    return $self->_load_stats( $params );
  }
  elsif($params && $params->{component} && $params->{component} eq 'database') {
    # HASHREF
    return $self->_database_stats( $params );
  }
  elsif($params && $params->{component} && $params->{component} eq 'cluster') {
    # HASHREF
    return $self->_cluster_stats( $params );
  }

  # default returns all
  my %ret = (
    cluster => $self->_cluster_stats( $params ),
    database => $self->_database_stats( $params ),
    queue => $self->_load_stats( $params )
  );
  return \%ret;
}
##### UPDATE
##### LOADID/LOAD-SOURCE
sub loadid_information {
  my $self = shift;
  my $loadid = shift;
  my $c = $self->{c};
  my $dbh = $self->dbh;
  my $tbl = ($c->config->{remote}) ? 't_client_load_process' : 't_load_process';

  my %ret = ( );
  my $sth = $dbh->prepare( "select * from reporting.$tbl where load_id = ?" );
  $sth->execute( $loadid );

  while( my $ref = $sth->fetchrow_hashref() ) {
    my %load = ();
    foreach my $key ( keys %{$ref} ) { $load{$key} = $ref->{$key}; }

    # don't confuse end user ...
    delete $load{server_name};
    delete $load{load_process_id};

    $ret{load} = \%load;
  }
  $sth->finish();
  return \%ret;
}


#### REMOTE CLIENT
sub loads_pending {
  my $self = shift;
  my $max_client_load_id = shift;
  my $limit = shift || 10;
  my @excluded_load_sources = ();

  my @load_info = ();

  my $c = $self->{c}; # for logging

  if( !@excluded_load_sources || !scalar( @excluded_load_sources ) ) {
    push(@excluded_load_sources, "XXX" );
  }

$c->log->debug( "EXCLUDED: " . Dumper(\@excluded_load_sources) );

  my @ls_params;
  my $sql;
  my $ls_where;
  foreach my $ls ( @excluded_load_sources ) { push( @ls_params, '?' ); }
  $ls_where = sprintf( " and load_source not in (%s)", join(",", @ls_params) );

  my $sql =
      "select *
       from reporting.t_client_extract_process
       where load_id > ?
         and running_status='complete'
         and completed_status='success'
         and ndocs > 0
         and url is not null
         $ls_where
       order by load_id limit ?";

$c->log->debug( "UPDATE SQL: $sql" );

  my $sth = $self->dbh->prepare( $sql );
  $sth->execute($max_client_load_id, @excluded_load_sources, $limit );

  while(my $ref = $sth->fetchrow_hashref()) {
    my %available_load_information = ();
    foreach my $key (keys %{$ref}) { $available_load_information{$key} = $ref->{$key}; }
    $available_load_information{session_id} = sprintf("%012d-%s", $ref->{load_id}, $ref->{load_source});
    push(@load_info, \%available_load_information);
  }
  $sth->finish();
  return \@load_info;
}



# RETRUN VALUES
#   -1      There is no connection to the database at all (e.g. after disconnect)
#   -2      An unknown transaction status was returned (e.g. after forking)
#   -3      The handle exists, but no data was returned from a test query.
sub ping {
  my $self = shift;
  my $dbh = $self->dbh;
  my $ret = $dbh->pg_ping();
  if($ret < 1) { return 0; }
  return $ret;
}

sub ACCEPT_CONTEXT {
  my ($self, $c, @args) = @_;
  $self = bless( { %{$self}, c => $c }, ref($self) );
  return $self;
}


__PACKAGE__->config(
  AutoCommit => q{1},
  pg_enable_utf8 => qw{1},
  pg_server_prepare => qw{0},

);

1;

__END__
