package Alexandria::CDWS::Model::Inventory;

use strict;
use base 'Catalyst::Model::DBI';

use XML::LibXML;
use Data::Dumper;

sub getdoc {
  my $self = shift;
  my $ucid = shift;
  my $dbh = $self->dbh;

  my $sql = "select p.publication_id,
                    p.application_id,
                    pt.product_code,
                    pt.exchange_date,
                    s.source_file,
                    rt.raw_text_id,
                    rt.raw
             from nrmdata.t_publications as p
               inner join nrmdata.t_publication_text as pt
                 on (pt.publication_id=p.publication_id)
               inner join nrmdata.t_raw_text as rt
                 on (pt.raw_text_id=rt.raw_text_id)
               inner join nrmdata.t_source_files as s
                 on (pt.source_file_id=s.source_file_id)
             where p.publication_uid=?
             order by pt.product_code,pt.exchange_date";

  my $sth = $dbh->prepare( $sql );
  $sth->execute( $ucid );
  my ($xml) = $sth->fetchrow_array();
  $sth->finish();
  return $xml;
}


sub fetch_specific_raw_text {
  my $self = shift;
  my $tbl = shift;
  my $raw_text_col_id = shift;
  my $raw_text_col = shift;
  my $raw_text_id = shift;

  my $dbh = $self->dbh;

  $dbh->{LongReadLen} = 10485760;
  $dbh->{LongTruncOk} = 0;

  my $sql = "select $raw_text_col from $tbl where $raw_text_col_id = ?";
  my $sth = $dbh->prepare( $sql );
  $sth->execute($raw_text_id);
  my ($xml) = $sth->fetchrow_array();
  $sth->finish();
  return $xml;
}

sub get_source_file_data {
  my $self = shift;
  my $ucid = shift;

  my $dbh = $self->dbh;
  my $sth = $dbh->prepare('select * from nrmdata.f_get_source_files(?)');
  $sth->execute($ucid);

  my @data = ();
  while(my $ref = $sth->fetchrow_hashref()) {
    my %h = ();
    foreach my $k (keys %{$ref}) {
      $h{$k} = $ref->{$k};
    }
    push(@data, \%h);
  }
  $sth->finish();
  return \@data;
}

sub get_inventory_xml {
  my $self = shift;
  my $ucid = shift;

  my $p = XML::LibXML->new();
  my $ret = XML::LibXML::Document->new('1.0');
  my $root = XML::LibXML::Element->new('inventory-items');
  $ret->setDocumentElement($root);

  my $dbh = $self->dbh;
  my $sth = $dbh->prepare( "select * from nrmdata.f_get_all_xml(?)" );
  $sth->execute($ucid);

  while(my $ref = $sth->fetchrow_arrayref()) {
    my $dom = $p->parse_string($ref->[0]);
    $root->addChild($dom->documentElement());
  }
  $sth->finish();
  return $ret;
}

  
sub trace_ucid {
  my $self = shift;
  my $ucid = shift;
  my $filedata = $self->_get_source_file_data($ucid);
  return $filedata;
}

sub ping {
  my $self = shift;
  my $dbh = $self->dbh;
  my $ret = $dbh->pg_ping();
  if($ret < 1) { return 0; }
  return $ret;
}

__PACKAGE__->config(
  AutoCommit => q{1},
);

1;

__END__

