package Alexandria::CDWS::Model::MSSQL;

use strict;
use base 'Catalyst::Model::DBI';

use Data::Dumper;

sub test {
  my $self = shift;
  my $ucid = shift;
  my $ret;

  my $dbh = $self->dbh;
  my $sth = $dbh->prepare('select 1');
  $sth->execute();
  while(my $ref = $sth->fetchrow_arrayref()) {
    $ret = $ref->[0];
  }
  return $ret;
}

sub get_raw_xml {
  my ($self, $ucid) = @_;
  my $dbh = $self->dbh;

  $dbh->{LongReadLen} = 10485760;
  $dbh->{LongTruncOk} = 0;

  my $sth = $dbh->prepare("select * from patents.dbo.f_buildPubXML(patents.dbo.f_getPubIdFromUcid(?), null)");
  $sth->execute($ucid);
  my ($xml) = $sth->fetchrow_array();
  $sth->finish();
  return $xml;
}
  
sub get_load_info_by_ucid {
  my ($self, $ucid) = @_;
  my @rows = ();
  my $dbh = $self->dbh;
  my $sth = $dbh->prepare("select *
                           from patents.dbo.v_publications
                           where pub_pub_id=patents.dbo.f_getPubIDfromUCID(?)");
  $sth->execute($ucid);
  while (my $ref = $sth->fetchrow_hashref()) {
    my %s = ();
    foreach my $k (keys %{$ref}) { $s{$k} = $ref->{$k}; }
    push(@rows, \%s);
  }
  return \@rows;
}

sub _get_file_loads {
  my $self = shift;
  my $loadid = shift;
  my $dbh = $self->dbh;
  my $sth = $dbh->prepare("select * from patents.dbo.t_file_loads where load_id=?");
  $sth->execute($loadid);
  my %h = ();
  while(my $ref = $sth->fetchrow_hashref()) {
    foreach my $k (keys %{$ref}) {
      $h{$k} = $ref->{$k};
    }
  }
  return \%h;
}

sub get_system_history_by_ucid {
  my ($self, $ucid) = @_;
  my %load_source_data = ();
  
  my $dbh = $self->dbh;
  my $sth = $dbh->prepare("select load_source, create_load_id, modified_load_id, deleted_load_id
                           from patents.dbo.t_publication_sources
                           where pub_id=patents.dbo.f_getPubIDfromUCID(?)
                           order by modified_load_id");
  $sth->execute($ucid);
  while (my $ref = $sth->fetchrow_hashref()) {
    my %load_ids = (
      create_load_id => $ref->{create_load_id},
      modified_load_id => $ref->{modified_load_id},
      deleted_load_id => $ref->{deleted_load_id}
    );
    $load_source_data{$ref->{load_source}} = \%load_ids;
  }
  $sth->finish();

#print STDERR Dumper(\%load_source_data);

  my @file_load_data = ();
  foreach my $source (keys %{load_source_data}) {
    my $ls = $load_source_data{$source};
    push(@file_load_data, $self->_get_file_loads($ls->{create_load_id}));
    if($ls->{create_load_id} != $ls->{modified_load_id}) {
      push(@file_load_data, $self->_get_file_loads($ls->{modified_load_id}));
    }
    if(defined($ls->{deleted_load_id})) {
      push(@file_load_data, $self->_get_file_loads($ls->{deleted_load_id}));
    }
  }
  my @loads = sort { $b->{load_id} <=> $a->{load_id} } @file_load_data;
# print STDERR Dumper(\@loads);
  return \@loads;
}

sub get_system_history_by_load_id {
  my ($self, $loadid) = @_;
  my $dbh = $self->dbh;
  my %s = ();
  my $sth = $dbh->prepare("select * from patents.dbo.t_file_loads where load_id=?");
  $sth->execute($loadid);
  while(my $ref = $sth->fetchrow_hashref()) {
    foreach my $k (keys %{$ref}) { $s{$k} = $ref->{$k}; }
  }
  $sth->finish;
  return \%s;
}

sub forward_citations {
  my ($self, $ucid) = @_;
  my @fcites = ();

  my $dbh = $self->dbh;
  my $sth = $dbh->prepare("select distinct(patents.dbo.f_getUCIDfromPubID(pub_id))
                           from t_pub_citations
                           where ref_pub_id=patents.dbo.f_getPubIdFromUcid_nd(?)");
  $sth->execute($ucid);
  while(my $ref = $sth->fetchrow_arrayref()) {
    push(@fcites, $ref->[0]);
  }
  $sth->finish();
  return \@fcites;
}

sub extended_inpadoc_family {
  my ($self, $ucid) = @_;
  my @family = ();

  my $dbh = $self->dbh;
  my $sth = $dbh->prepare(
            "select patents.dbo.f_getUCIDfromPubID(x.publication_id) as a
             from patents.dbo.t_publications t
               cross apply patents.dbo.f_buildExtendedInpadocFamily( t.pub_id ) as x
             where t.pub_id = patents.dbo.f_getPubIDfromUCID(?)
             order by a"
  );
  $sth->execute($ucid);
  while(my $ref = $sth->fetchrow_arrayref()) {
    push(@family, $ref->[0]);
  }
  $sth->finish();
  return \@family;

}

__PACKAGE__->config(
  AutoCommit => q{1},
);

1;

__END__

