package Alexandria::CDWS::Model::Report;

use Moose;
extends qw(Alexandria::CDWS::Model::DBIx::Connector);
use Data::Dumper;

sub test {
  my $self = shift;
  my $dbh = $self->dbh;
}

sub columns {
  my $self = shift;
  my $params = shift;
  my $dbh = $self->dbh;

  my $c = $self->{c};
  my $user = $c->user;
$c->log->debug( "LEVEL: " . $user->subscription_level );
  my %ret = ();
  my @cols = ();
  eval {
    my $sth = $dbh->prepare('select csv_column from reports.t_csv_columns order by csv_ordinal');
    $sth->execute();
    while(my $ref = $sth->fetchrow_arrayref() ) {
      push( @cols, $ref->[0] );
    }
    $sth->finish();
  };
  if($@) {
    $ret{status} = 'error';
    $ret{message} = 'Failed to aquire available report columns';
    $ret{debug} = $@;
    return \%ret;
  }
  $ret{status} = 'success';
  $ret{columns} = \@cols;
  return \%ret;
}

sub listex {
  my $self = shift;
  my $params = shift;
  my $dbh = $self->dbh;

  my $c = $self->{c};

  my $user = $c->user;
#$c->log->debug( __PACKAGE__ . ':listex()' );
#$c->log->debug( Dumper($params) );

  my $limit = ($params->{limit}) ? "$params->{limit}" : 'all';
  my $offset = ($params->{offset}) ? "$params->{offset}" : '0';
  my %ret = ();
  my @reports = ();
  eval {
    my $sth = $dbh->prepare(
      "select t2.user_id, t1.*
       from reports.t_reports t1
         inner join reports.t_user_reports t2
           on (t1.report_id=t2.report_id)
       where t2.user_id=? order by t1.report_id desc
       limit $limit
       offset $offset"
    );
    $sth->execute( $user->id );
    while( my $ref = $sth->fetchrow_hashref() ) {
      my %s = ();
      foreach my $key ( keys %{$ref} ) { $s{$key} = $ref->{$key}; }

      my $id = $ref->{tracker_id};
      delete $s{tracker_id};
      $s{id} = $id;
      push(@reports, \%s);
    }
  };
  if($@) {
    $ret{status} = 'error';
    $ret{message} = 'Failed to list reports';
    $ret{debug} = $@;
    return \%ret;
  }
#
#  eval {
#    my $sth = $dbh->prepare(
#      "select t2.*
#       from reports.t_reports t0
#         inner join reports.t_report_process t1
#           on (t0.report_id=t1.report_id)
#         inner join reports.t_process t2
#           on (t1.process_id=t2.process_id)
#       where t0.report_id=?");
#    foreach my $r ( @reports ) {
#      $r->{status} = 'complete';
#      $sth->execute( $r->{report_id} );
#      while( my $ref = $sth->fetchrow_hashref() ) {
#        if( $ref->{running_status} != 'complete' ) {
##          $r->{status} = $ref->{running_status};
#        }
#        push( @{$r->{processes}}, $ref );
#      }
#    }
#  };
#  if( $@ ) {
#    $ret{status} = 'error';
#    $ret{message} = 'Failed to list reports';
#    $ret{debug} = $@;
#    return \%ret;
#  }
  $ret{status} = 'success';
  $ret{reports} = \@reports;
  return \%ret;
}

sub list {
  my $self = shift;
  my $params = shift;
  my $dbh = $self->dbh;

  my $c = $self->{c};

  my $user = $c->user;
$c->log->debug( __PACKAGE__ . ':list()' );
$c->log->debug( Dumper($params) );

  my $limit = ($params->{limit}) ? "$params->{limit}" : 'all';
  my $offset = ($params->{offset}) ? "$params->{offset}" : '0';
  my %ret = ();
  my @reports = ();
  eval {
    my $sth = $dbh->prepare(
      "select * from reports.v_reports
         where user_id=? order by report_id desc
         limit $limit
         offset $offset"
    );
    $sth->execute( $user->id );
    while( my $ref = $sth->fetchrow_hashref() ) {
      my %s = ();
      foreach my $key ( keys %{$ref} ) { $s{$key} = $ref->{$key}; }

      my $id = $ref->{tracker_id};
      delete $s{tracker_id};
      $s{id} = $id;
      push(@reports, \%s);
    }
  };
  if($@) {
    $ret{status} = 'error';
    $ret{message} = 'Failed to list reports';
    $ret{debug} = $@;
  }
  else {
    $ret{status} = 'success';
    $ret{reports} = \@reports;
  }
  return \%ret;
}

sub create {
  my $self = shift;
  my $params = shift;
  my $dbh = $self->dbh;

  my $c = $self->{c};

  my $user = $c->user;

$c->log->debug( __PACKAGE__ . ':create()' );
$c->log->debug( Dumper($params) );

  my $report_id = -1;
  my %ret = ( status => 'success' ); # optimisim ...

  eval {
    my $sth = $dbh->prepare( "select reports.f_report_create(?,?,?,?,?,?,?,?,?)");
    $sth->execute(
      $user->id,
      $params->{name},
      $params->{type},
      $user->subscription_level,
      $params->{source},
      $params->{criteria},
      $self->{solrurl},
      $params->{solrparams},
      $params->{csvopts}
    );
    ($report_id) = $sth->fetchrow_array();
  };
  if($@) {
    $c->log->error( $@ );
    $ret{status} = 'error';
    $ret{message} = 'Failed to create report';
    $ret{debug} = $@;
  }
  else {
    my $sth = $dbh->prepare("select tracker_id from reports.t_user_reports where report_id=?");
    $sth->execute( $report_id );
    my ($id) = $sth->fetchrow_array();
    $sth->finish();

    $ret{status} = 'success';
    $ret{report} = { id => $id };
  }
  return \%ret;
}

sub queue {
  my $self = shift;
  my $params = shift;
  my $dbh = $self->dbh;

  my $c = $self->{c};

  my $user = $c->user;

$c->log->debug( __PACKAGE__ . ':queue()' );
$c->log->debug( Dumper($params) );

  my $process_id = -1;

  my %ret = ( status => 'success' ); # optimisim ...

  eval {
    my $sth = $dbh->prepare( "select reports.f_report_queue( reports.f_tracker2id(?),?,?,?,?,?,?,?)" );
    $sth->execute(
      $params->{tracker_id},
      $params->{nthreads},
      $params->{batchsize},
      $params->{batchprefix},
      $params->{package_type},
      $params->{package_sfx},
      $params->{package_compression},
      $params->{priority}
    );
    ($process_id) = $sth->fetchrow_array();
  };
  if($@) {
    $c->log->error( $@ );
    $ret{status} = 'error';
    $ret{message} = 'Failed to queue report';
    $ret{debug} = $@;
  }
  else {
    $ret{status} = 'success';
    $ret{queue} = { id => $process_id };
  }
  return \%ret;
}

sub detail {
  my $self = shift;
  my $params = shift;
  my $dbh = $self->dbh;

  my $c = $self->{c};

  my $user = $c->user;

$c->log->debug( __PACKAGE__ . ':details()' );
$c->log->debug( Dumper($params) );

  my %ret = ();
  my %report_detail = ();
  eval {
    my $sth = $dbh->prepare('select * from reports.v_reports where user_id=? and report_id=reports.f_tracker2id(?)');
    $sth->execute( $user->id, $params->{tracker_id} );
    while( my $ref = $sth->fetchrow_hashref() ) {
      foreach my $key ( keys %{$ref} ) { $report_detail{$key} = $ref->{$key}; }
    }
  };
  if($@) {
    $ret{status} = 'error';
    $ret{message} = 'Failed to list reports';
    $ret{debug} = $@;
  }
  else {
    $ret{status} = 'success';
    $ret{report} = \%report_detail;
  }
  return \%ret;
}

sub status {
  my $self = shift;
  my $params = shift;
  my $dbh = $self->dbh;

  my $c = $self->{c};

  my $user = $c->user;

$c->log->debug( __PACKAGE__ . ':status()' );
$c->log->debug( Dumper($params) );

  my %ret = ();
  my %report_status = ();
  eval {
    my $sth = $dbh->prepare(
      "select
         running_status,
         to_char(entered_stamp, 'YYYY-MM-DD HH24:MI:SS') as entered_stamp,
         to_char(completed_stamp, 'YYYY-MM-DD HH24:MI:SS') as completed_stamp,
         completed_status,
         completed_url,
         completed_error,
         ndocs,
         ndocs_processed
       from reports.v_reports
       where user_id=? and tracker_id=?"
    );
    $sth->execute( $user->id, $params->{tracker_id} );
    while( my $ref = $sth->fetchrow_hashref() ) {
      foreach my $key ( keys %{$ref} ) { $report_status{$key} = $ref->{$key}; }
      if( !$ref->{report_ndocs} || !$ref->{report_ndocs_processed} ) {
        $report_status{percent_complete} = sprintf("%.3f", 0 );
      }
      else {
        $report_status{percent_complete} = sprintf("%.3f",
          ( ( ( $ref->{report_ndocs_processed} ) ? $ref->{report_ndocs_processed} : 0
                 /
                ( $ref->{report_ndocs} ) ? $ref->{report_ndocs} : 0 ) * 100) );
      }
    }
  };
  if($@) {
    $ret{status} = 'error';
    $ret{message} = 'Failed to acquire report status';
    $ret{debug} = $@;
  }
  else {
    $ret{status} = 'success';
    $ret{report} = \%report_status;
  }
  return \%ret;
}

sub delete {
  my $self = shift;
  my $params = shift;

  my $dbh = $self->dbh;

  my $c = $self->{c};

  my $user = $c->user;

$c->log->debug( __PACKAGE__ . ':delete()' );
$c->log->debug( Dumper($params) );

  my %ret = ( status => 'success' ); # optimisim ...

  eval {
    my $sth = $dbh->prepare( "select reports.f_report_delete( reports.f_tracker2id(?) )" );
    $sth->execute( $params->{tracker_id} );
  };
  if($@) {
    $c->log->error( $@ );
    $ret{status} = 'error';
    $ret{message} = 'Failed to delete report';
    $ret{debug} = $@;
  }
  else {
    $ret{status} = 'success';
  }
  return \%ret;
}

sub ping {
  my $self = shift;
  my $dbh = $self->dbh;
  my $ret = $dbh->pg_ping();
  if($ret < 1) { return 0; }
  return $ret;
}

sub ACCEPT_CONTEXT {
  my ($self, $c, @args) = @_;
  $self = bless( { %{$self}, c => $c }, ref($self) );
  return $self;
}

__PACKAGE__->config(
  AutoCommit => q{1},
);

1;

__END__

