package Alexandria::CDWS::Model::Users;

use Moose;
extends qw(Alexandria::CDWS::Model::DBIx::Connector);

sub log_statistics {
  my $self = shift;
  my $params = shift;
  my $c = $self->{c};

  my $sql = "insert into admin.t_user_statistics
                ( user_id, address, method, path, parameters, response_code,
                  response_length, user_agent, request_time, source_server )
               values ( admin.f_name2id(?), ?, ?, ?, ?, ?, ?, ?, ?, ?  )";
  my $dbh = $self->dbh;
  eval {
    my $sth = $dbh->prepare( $sql );
    $sth->execute(
      $params->{name},
      $params->{address},
      $params->{method},
      $params->{path},
      $params->{parameters},
      $params->{response_code},
      $params->{response_length},
      $params->{user_agent},
      $params->{request_time},
      $params->{source_server}
    );
    $sth->finish();
  };
  if( $@ ) { $c->log->error( $@ ); }
}

sub fetch_general_statistics {
  my ( $self, $span ) = @_;
  my $table;
  if( !$span ) { $table = 'admin.v_user_statistics'; }
  elsif( $span eq 'week' ) { $table = 'admin.v_user_statistics_last_week'; }
  elsif( $span eq 'month' ) { $table = 'admin.v_user_statistics_last_month'; }

  my $c = $self->{c};
  my $sql = "select count(*) as nreqs, user_name, path
             from $table
             group by user_name, path
             order by user_name, nreqs desc, path";
  my $dbh = $self->dbh;
  my $sth = $dbh->prepare( $sql );
  my @stats = ();
  eval {
    $sth->execute( );
    while( my $ref = $sth->fetchrow_hashref() ) {
      push( @stats, { nreqs => $ref->{nreqs}, user_name => $ref->{user_name},  path => $ref->{path} } );
    }
    $sth->finish();
  };
  if( $@ ) { $c->log->error( $@ ); }
  return \@stats;
}

sub fetch_statistics {
  my ( $self, $id, $span ) = @_;
  my $table;
  if( !$span ) { $table = 'admin.v_user_statistics'; }
  elsif( $span eq 'week' ) { $table = 'admin.v_user_statistics_last_week'; }
  elsif( $span eq 'month' ) { $table = 'admin.v_user_statistics_last_month'; }

  my %statistics = ();
  my %meta = ();
  my @reqs = ();

  my $c = $self->{c};

  ### requests
  my $sql = "select count(*) as nreqs, path
             from $table
             where user_id=?
             group by path
             order by nreqs desc, path";
  my $dbh = $self->dbh;
  my $sth = $dbh->prepare( $sql );

  eval {
    $sth->execute( $id );
    while( my $ref = $sth->fetchrow_hashref() ) {
      push( @reqs, { nreqs => $ref->{nreqs}, path => $ref->{path} } );
    }
    $sth->finish();
  };
  if( $@ ) { $c->log->error( $@ ); }

  $statistics{requests} = \@reqs;

  ### meta information
  $sth = $dbh->prepare( "select max(request_stamp)
                         from $table where user_id=?
                         limit 1"
  );

  eval {
    $sth->execute( $id );
    my ( $stamp ) = $sth->fetchrow_array();
    $meta{last_access} = $stamp;
    $sth->finish();
  };
  if( $@ ) { $c->log->error( $@ ); }

  $statistics{meta} = \%meta;
  return \%statistics;
}


sub _list {
  my $self = shift;
  my $sql = shift;
  my $params = shift;

  my @ret = ();

  my $dbh = $self->dbh;
  my $sth = $dbh->prepare( $sql );
  $sth->execute();
  while( my $ref = $sth->fetchrow_hashref() ) {
    if($params && $params->{all}) {
      push(@ret, $ref);
    }
    else {
      my %h = ();
      foreach my $key ( keys %{$ref} ) {
        if($ref->{$key}) { $h{$key} = $ref->{$key}; }
      }
      push(@ret, \%h);
    }
  }
  $sth->finish();
  return \@ret;
}

sub user_list {
  return $_[0]->_list( 'select * from admin.t_users where passwd is not null order by user_name', $_[1] );
}

sub group_list {
  return $_[0]->_list( 'select * from admin.t_groups order by group_name', $_[1] );
}

sub privilege_list {
  return $_[0]->_list( 'select * from admin.t_privileges order by privilege_name', $_[1] );
}

sub subscription_level_list {
  return $_[0]->_list( 'select * from admin.t_subscription_levels order by subscription_level_id', $_[1] );
}

sub ip_address_list {
  return $_[0]->_list( 'select * from admin.t_ip_addresses order by address', $_[1] );
}

sub _info {
  my $self = shift;
  my $sql = shift;
  my $id = shift;
  my $dbh = $self->dbh;
  my $sth = $dbh->prepare( $sql );
  $sth->execute( $id );
  my ($ref) = $sth->fetchrow_hashref();
  $sth->finish();
  return $ref;
}

sub user_info {
  return $_[0]->_info( 'select * from admin.f_user_information (?)', $_[1] );
}

sub group_info {
  return $_[0]->_info( 'select * from admin.f_group_information (?)', $_[1] );
}

sub privilege_info {
  return $_[0]->_info( 'select * from admin.f_privilege_information (?)', $_[1] );
}

sub user_add {
  my $self = shift;
  my $user = shift; # anonymous hash

  my $dbh = $self->dbh;
  my $sth = $dbh->prepare( 'select admin.f_add_user(?,?,?,?,?,?,?,?,?)' );
  $sth->execute(
    $user->{user_name},
    $user->{passwd},
    $user->{email},
    $user->{phone},
    $user->{descr},
    $user->{groups},
    $user->{privileges},
    $user->{subscription_level},
    $user->{ip_addresses}
  );
  my ($id) = $sth->fetchrow_array();

  $user->{user_id} = $id;

  return $user;
}

sub group_add {
  my $self = shift;
  my $group = shift; # anonymous hash

  my $dbh = $self->dbh;
  my $sth = $dbh->prepare( 'select admin.f_add_group(?,?,?)' );
  $sth->execute(
    $group->{group_name},
    $group->{group_descr},
    $group->{privileges}
  );
  my ($id) = $sth->fetchrow_array();

  $group->{group_id} = $id;

  return $group;
}

sub user_delete {
  my $self = shift;
  my $id = shift;
  my $dbh = $self->dbh;
  my $sth = $dbh->prepare( 'select admin.f_delete_user(?)' );
  $sth->execute( $id );
  my ($ret) = $sth->fetchrow_array();
  return $ret;
}

sub group_delete {
  my $self = shift;
  my $id = shift;
  my $dbh = $self->dbh;
  my $sth = $dbh->prepare( 'select admin.f_delete_group(?)' );
  $sth->execute( $id );
  my ($ret) = $sth->fetchrow_array();
  return $ret;
}

sub user_update {
  my $self = shift;
  my $user = shift;
  my $dbh = $self->dbh;

  my @sets = ( );
  my @params = ( );

  foreach my $k ( keys %{$user} ) {
    if($user->{$k} && $k !~ /(user_id|ip_addresses|privileges|groups|subscription_level)/) {
      push(@sets, "$k = '$user->{$k}'");
    }
  }

  my $status;
  my $c = $self->{c};

  eval {
    if(scalar(@sets)) {
      my $sql = sprintf( "update admin.t_users set %s where user_id=%d", join(',', @sets), $user->{user_id} );
      $status = $dbh->do( $sql );
      $c->log->debug(__PACKAGE__ . ": status=$status");
    }

    if($user->{ip_addresses}) {
      $dbh->do("delete from admin.t_user_ip_addresses where user_id=$user->{user_id}");
      foreach my $ip ( @{$user->{ip_addresses}} ) {
        $status = $dbh->do("select admin.f_ip_add_member( admin.f_ip2id(\'$ip\'), $user->{user_id} )");
      }
    }
    if($user->{privileges}) {
      $dbh->do("delete from admin.t_user_privileges where user_id=$user->{user_id}");
      foreach my $prv ( @{$user->{privileges}} ) {
        $status = $dbh->do("select admin.f_privilege_add_member( admin.f_privilege2id(\'$prv\'), $user->{user_id} )");
      }
    }
    if($user->{groups}) {
      $dbh->do("delete from admin.t_user_groups where user_id=$user->{user_id}");
      foreach my $grp ( @{$user->{groups}} ) {
        $status = $dbh->do("select admin.f_group_add_member( admin.f_group2id(\'$grp\'), $user->{user_id} )");
      }
    }
    if($user->{subscription_level}) {
      $dbh->do("delete from admin.t_user_subscription_levels where user_id=$user->{user_id}");
      $status = $dbh->do("select admin.f_subscription_level_add_member( admin.f_subscription_level2id(\'$user->{subscription_level}\'), $user->{user_id} )");
    }
  };
  if($@) { $c->log->warns("$@"); return undef; }

  return $self->user_info( $user->{user_id} );
}

sub group_update {
  my $self = shift;
  my $group = shift;
  my $dbh = $self->dbh;

  my @sets = ( );
  my @params = ( );

  my $c = $self->{c};

  my $status;

  foreach my $k ( keys %{$group} ) {
    if($group->{$k}) {
      push(@sets, "$k = '$group->{$k}'");
    }
  }

  eval {
    if(scalar(@sets)) {
      my $sql = sprintf( "update admin.t_groups set %s where group_id=%d", join(',', @sets), $group->{group_id} );
$c->log->debug("SQL: $sql");
      my %ret = ( status => $dbh->do( $sql ) );
      return \%ret;
    }
  };
  if($@) { warn "$@"; return undef; }

  return $self->group_info( $group->{group_id} );
}

sub add_to_group {
  my $self = shift;
  my $groups = shift; # array of groups
}




########### SESSIONS
sub session_list {
  my $self = shift;
  my $params = shift;
  my $dbh = $self->dbh;
  my $sth = $dbh->prepare(
    'select t1.session_id, t1.session_key, t1.session_name,
       t1.created_stamp, t1.modified_stamp, t1.notes
     from data.t_sessions as t1
       inner join data.t_user_sessions as t2
         on (t1.session_id=t2.session_id)
     where t2.user_id=? and t1.deleted_stamp is null'
  );
  $sth->execute( $params->{user_id} );
  my @ret = ();
  while (my $ref = $sth->fetchrow_hashref() ) {
    push(@ret, $ref);
  }
  $sth->finish();
  return \@ret; # array of sessions
}
sub session_new {
  my $self = shift;
  my $params = shift;
  my $dbh = $self->dbh;
  my $sth = $dbh->prepare( 'select data.f_session_new(?,?,?)' );
  $sth->execute( $params->{user_id},
                ($params->{session_name}) ? $params->{session_name} : undef,
                ($params->{session_note}) ? $params->{session_note} : undef
  );
  my ($ret) = $sth->fetchrow_array();
  return $ret; # session_key
}

sub session_update {
  my $self = shift;
  my $params = shift;
  my $dbh = $self->dbh;
  my $sth = $dbh->prepare( 'select data.f_session_update(?,?,?)' );
  $sth->execute( $params->{user_id}, $params->{skey}, $params->{sdata} );
  my ($ret) = $sth->fetchrow_array();
  return $ret; # boolean
}

sub session_delete {
  my $self = shift;
  my $params = shift;
  my $dbh = $self->dbh;
  my $sth = $dbh->prepare( 'select data.f_session_delete(?,?)' );
  $sth->execute( $params->{user_id}, $params->{skey} );
  my ($ret) = $sth->fetchrow_array();
  return $ret; # boolean
}

sub session_fetch_meta {
  my $self = shift;
  my $params = shift;
  my $dbh = $self->dbh;
  my $sth = $dbh->prepare( 'select * from data.t_sessions where session_key = ?' );
  $sth->execute( $params->{skey} );

  my %ret = ();
  while( my $ref = $sth->fetchrow_hashref() ) {
    foreach my $k ( keys %{ $ref } ) { $ret{$k} = $ref->{$k}; }
  }
  return \%ret;
}

sub session_fetch {
  my $self = shift;
  my $params = shift;
  my $dbh = $self->dbh;
  my $sth = $dbh->prepare( 'select data.f_session_fetch(?,?)' );
  $sth->execute( $params->{user_id}, $params->{skey} );
  my ($ret) = $sth->fetchrow_array();
  return $ret; # the session
}

###############



sub ping {
  my $self = shift;
  my $dbh = $self->dbh;
  my $ret = $dbh->pg_ping();
  if($ret < 1) { return 0; }
  return $ret;
}

sub ACCEPT_CONTEXT {
  my ($self, $c, @args) = @_;
  $self = bless( { %{$self}, c => $c }, ref($self) );
  return $self;
}


__PACKAGE__->config(
  AutoCommit => q{1},
);

1;

__END__

