package Alexandria::CDWS::Model::Zabbix;

use utf8;
use strict;
use warnings;


use Moose;
use Moose::Util::TypeConstraints;

use LWP::UserAgent;
use HTTP::Headers;
use HTTP::Request;
use HTTP::Request::Common;
use URI;
use JSON;
use Data::Dumper;
extends 'Catalyst::Model';

has 'url' => (
  is      => 'ro',
  isa     => 'URI',
  default => sub { URI->new( 'http://10.234.1.67/zabbix/api_jsonrpc.php' ) }
);

has 'agent' => (
  is => 'ro',
  isa => 'Object',
  default => sub {
    LWP::UserAgent->new(
      agent => 'cdws-v2.0-zabbix',
      default_headers => HTTP::Headers->new(
        'Accept' => 'application/json',
        'Content-Type' => 'application/json'
      )
    );
  }
);

has 'json' => (
  is => 'ro',
  isa => 'Object',
  default => sub {
    JSON->new();
  }
);

sub _post {
  my ($self, $obj) = @_;
  my $json = $self->json->encode( $obj );
  $self->{c}->log->debug( $json );
  my $resp = $self->agent->request(
    POST( $self->url, 'Content-Type' => 'application/json', Content => $json )
  );
#  $self->{c}->log->debug( Dumper ($resp ) );
  return $resp;
}

sub _auth {
  my $self = shift;
  my $c = $self->{c};

  my %obj = (
    "jsonrpc" => "2.0",
    "method" => "user.login",
    "params" => {
       "user" => "david",
       "password" => ""
    },
    "id" => 1
  );
  my $ret = $self->_post( \%obj );
# {"jsonrpc":"2.0","result":"a61947a9af76a575228bfccfab9c3522","id":1}

  my $s;
  eval {
    $s = $self->json->decode( $ret->content );
#$c->log->debug( Dumper( $s ) );
  };
  if( $@ ) {
    $c->log->error( "Zabbix failure: $@" );
  }
  return $s;
}
  
sub execute {
  my ($self, $json) = @_;
  my $c = $self->{c};

  my $auth = $self->_auth();

#$self->{c}->log->debug( Dumper ($auth ) );

  if( !$auth ) { return undef; }

  my $obj = $self->json->decode( $json );
  $obj->{auth} = $auth->{result};
  $obj->{id} = $auth->{id};

  my $ret = $self->_post( $obj );

$self->{c}->log->debug( Dumper ($ret ) );

  my $s;
  eval {
    $s = $self->json->decode( $ret->content );
  };
  if( $@ ) {
    $c->log->error( "Zabbix failure: $@" );
  }
  return $s;
}

sub ACCEPT_CONTEXT {
  my ($self, $c, @args) = @_;
  $self = bless( { %{$self}, c => $c }, ref($self) );
  return $self;
}

__PACKAGE__->config();


=head1 NAME

Alexandria::Model::Solr - Catalyst Model

=head1 DESCRIPTION

Alexandria::Model::Solr is a simple sub-class of Catalyst::Model.

=head1 AUTHOR

David Santamauro E<lt>david.santamauro@fairviewresearch.comE<gt>

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->meta->make_immutable;

1;

