package Catalyst::Action::Serialize::XML::Alexandria::XDWHStats;

use Moose;
use namespace::autoclean;
use Data::Dumper;
extends 'Catalyst::Action';

sub _cluster_xml {
  my $content = shift;
  my $cluster = XML::LibXML::Element->new('cluster');

  foreach my $att ( sort keys %{$content} ) {
    $cluster->setAttribute($att, $content->{$att});
  }
  return $cluster;
}

sub _database_xml {
  my $content = shift;
  my $database = XML::LibXML::Element->new('database');

  foreach my $att ( sort keys %{$content} ) {
    $database->setAttribute($att, $content->{$att});
  }
  return $database;
}

sub _queue_xml {
  my $content = shift;
  my $queue = XML::LibXML::Element->new('queue');

  foreach my $e ( @{$content} ) {
    my $load = XML::LibXML::Element->new('load');
    foreach my $att ( sort keys %{$e} ) {
      $load->setAttribute($att, $e->{$att});
    }
    $queue->addChild($load);
  }
  return $queue;
}

sub execute {
  my $self = shift;
  my ($controller, $c) = @_;

  eval { require XML::LibXML; };
  if($@) {
    $c->log->warning("Could not load XML::Serializer, refusing to serialize: $@");
    return;
  }

  my $stash_key = (
          $controller->{'serialize'} ?
              $controller->{'serialize'}->{'stash_key'} :
              $controller->{'stash_key'}
      ) || 'rest';
  my $content = $c->stash->{$stash_key};

  if(!$content) {
    $c->response->status(500);
    $c->log->error('database statistics content empty!');
    return;
  }

  # $c->log->debug(__PACKAGE__ . " CONTENT: " . Dumper($content) );

  return 1 if $c->req->method eq 'HEAD';
  return 1 if length( $c->response->body );
  return 1 if scalar @{ $c->error };
  return 1 if $c->response->status =~ /^(?:204|3\d\d)$/;

  # we know what we are building
  my $dom = XML::LibXML::Document->new('1.0');
  my $root = XML::LibXML::Element->new('response');
  $dom->setDocumentElement($root);

  # grab comonent if any ...
  my $comp = $c->request->parameters->{component};
  if($comp && $comp eq 'cluster') {
    $root->addChild(_cluster_xml($content));
  }
  elsif($comp && $comp eq 'database') {
    $root->addChild(_database_xml($content));
  }
  elsif($comp && $comp eq 'queue') {
    $root->addChild(_queue_xml($content));
  }
  else {
    # all of it
    $root->addChild( _cluster_xml($content->{cluster}) );
    $root->addChild( _database_xml($content->{database}) );
    $root->addChild( _queue_xml($content->{queue}) );
  }

  $c->response->output( $dom->toString(1) );

  return 1;
}

__PACKAGE__->meta->make_immutable;
1;

