package Catalyst::Action::Serialize::XML::Alexandria::XIDXStats;

use Moose;
use namespace::autoclean;
use Data::Dumper;
use XML::Simple;
use XML::XML2JSON;

extends 'Catalyst::Action';

sub execute {
  my $self = shift;
  my ($controller, $c) = @_;

  $c->response->status(415);
  $c->log->error('XML Output unsupported for index details');
  return;

  eval { require XML::LibXML; };
  if($@) {
    $c->log->warning("Could not load XML::Serializer, refusing to serialize: $@");
    return;
  }

  my $stash_key = (
          $controller->{'serialize'} ?
              $controller->{'serialize'}->{'stash_key'} :
              $controller->{'stash_key'}
      ) || 'rest';
  my $content = $c->stash->{$stash_key};

  if(!$content) {
    $c->response->status(500);
    $c->log->error('database statistics content empty!');
    return;
  }

  my $p = XML::LibXML->new();

  # $c->log->debug(__PACKAGE__ . " CONTENT: " . Dumper($content) );

  return 1 if $c->req->method eq 'HEAD';
  return 1 if length( $c->response->body );
  return 1 if scalar @{ $c->error };
  return 1 if $c->response->status =~ /^(?:204|3\d\d)$/;

  # we know what we are building
  my $dom = XML::LibXML::Document->new('1.0');
  my $root = XML::LibXML::Element->new('response');
  $dom->setDocumentElement($root);

  # grab component if any ...
  my $comp = $c->request->parameters->{component};
  if($comp && $comp eq 'cloud') {
    $root->addChild(_cloud_xml($content));
  }
  elsif($comp && $comp eq 'core') {
    # $c->log->debug(__PACKAGE__ . " CONTENT: " . Dumper($content->{core}) );
    $root->addChild( _core_xml($p, $content, $c) );
  }
  elsif($comp && $comp eq 'queue') {
    $root->addChild(_queue_xml($content));
  }
  else {
    # all of it
    $root->addChild( _cloud_xml($content->{cloud}) );
    $root->addChild( _core_xml($content->{core}) );
    $root->addChild( _queue_xml($content->{queue}) );
  }

  $c->response->output( $dom->toString(1) );

  return 1;
}

__PACKAGE__->meta->make_immutable;
1;

