package Catalyst::Action::Serialize::XML::Alexandria::XResponse;

use Moose;
use namespace::autoclean;
use Data::Dumper;
extends 'Catalyst::Action';


#####
# This method sets up a response for all
# XML content-type responses
# All Subclasses that want a standard:
#  <response ... > root element should have an execute()
# method that looks like:
#
# sub execute {
#   my ($self, $controller, $c) = @_;
#   my $xresp = $self->SUPER::execute( @_ );
#
#  where xresp is a hash
#  { dom => XML::LibXML::Document, content => response content }
#
#
sub execute {
  my $self = shift;
  my ($controller, $c) = @_;

#$c->log->debug( __PACKAGE__ . '::execute()' );

  eval { require XML::LibXML; };
  if($@) {
    $c->log->warn("Could not load XML::Serializer, refusing to serialize: $@");
    return;
  }

  # we grab the content here
  my $stash_key = (
          $controller->{'serialize'} ?
              $controller->{'serialize'}->{'stash_key'} :
              $controller->{'stash_key'}
      ) || 'rest';
  my $content = $c->stash->{$stash_key};


  # don't serialize:
  # HEAD methods
  return 1 if $c->req->method eq 'HEAD';

  # if there is content already in the response
  return 1 if length( $c->response->body );

  # if there is any kind of error
  return 1 if scalar @{ $c->error };


  # create standard response XML root
  my $dom = XML::LibXML::Document->new('1.0');
  my $root = XML::LibXML::Element->new('response');
# if($c->sessionid) { $root->setAttribute('sid', $c->sessionid); }
  $root->setAttribute('method', $c->request->path);

  $dom->setDocumentElement($root);

  if(!$content) {
    # this always warrants a warning
    $c->log->warning("Nothing to serialize");
    $root->setAttribute('status', 'unknown');
    $root->setAttribute('message', 'empty response');
  }

  $root->setAttribute('time', $content->{time} || "0.000");
  $root->setAttribute('status', $content->{status} || 'unknown');
  $root->setAttribute('message', $content->{message}) if $content->{message};

# $c->log->debug( Dumper($content) );

  ### DEBUG responses
  if( $c->request->parameters->{__debug} ) {
    my $debug = XML::LibXML::Element->new('debug');
    $debug->addChild(
       XML::LibXML::CDATASection->new( Dumper($c) )
    );

    $root->addChild( $debug );
  }

  my %ret = ( dom => $dom, content => $content );

# $c->log->debug( Dumper(\%ret) );

  return \%ret;
}

__PACKAGE__->meta->make_immutable;

1;

