package Catalyst::Action::Serialize::XML::Alexandria::XResponse::Custom::TWS;

use Moose;
use namespace::autoclean;
use Data::Dumper;
extends 'Catalyst::Action::Serialize::XML::Alexandria::XResponse';


sub execute {
  my $self = shift;
  my ($controller, $c) = @_;

  # builds the container response XML
  my $xresp = $self->SUPER::execute(@_);

#$c->log->debug( __PACKAGE__ . '::execute()' );
#$c->log->debug( Dumper($xresp) );

  # we know what we are building
  my $dom = $xresp->{dom};
  my $root = $dom->documentElement();

  my $data = $xresp->{content};
  if(!$data) { # setup already
    $c->response->output( $dom->toString(1) );
    return 1;
  }

#$c->log->debug( Dumper($data) );

  $root->setAttribute('status', $data->{status} );

  # tws-container { iid, lookup_status, confidence
  #   an => [
  #   ]
  # }
  # INPUT => [
  #   tws-container {
  #     lookup_status => "found|not-found|duplicate"
  #     confidence => "[0..100]"
  #     tws_applications => [
  #       {
  #       }
  #     ]
  #   },
  #   tws-container {
  #   }
  # ]

  my %docid_att_tbl = ( ucid => 'ucid', format => 'format',
                        country => 'country',
                        doc_number => 'doc-number', doc_number_original => 'doc-number-original',
                        date => 'date', kind => 'kind',
                        effective_grant_date => 'effective-grant-date'
  );

  foreach my $input ( sort keys %{ $data->{content} } ) {
    my $container = $data->{content}->{$input};
    my $tws = XML::LibXML::Element->new('tws-container');
    my %cont_att_tbl = ( 'iid' => 'iid', 'lookup_status' => 'lookup-status', 'confidence' => 'confidence' );
    foreach my $att ( sort keys %cont_att_tbl ) {
      $tws->setAttribute( $cont_att_tbl{ $att }, $container->{ $att } )
        if $container->{ $att };
    }

    if( $container->{duplicates} ) {
#$c->log->debug( Dumper( $container->{duplicates} ) );
      my $dups = XML::LibXML::Element->new('duplicates');
      foreach my $ucid ( @{ $container->{duplicates} } ) {
        my $dup = XML::LibXML::Element->new('duplicate');
        $dup->setAttribute( 'ucid', $ucid );
        $dups->addChild( $dup );
      }
      $tws->addChild( $dups );
    }

    foreach my $twsapp ( @{ $container->{tws_applications} } ) {
      my $twsappnode = XML::LibXML::Element->new( 'tws-application' );
      my %app_att_tbl = ( %docid_att_tbl,
        effective => 'effective', origin => 'origin', fam => 'fam',
        entity_status => 'entity-status',
        claims_total => 'claims-total', patent_status => 'patent-status',
        relation => 'relation', parent => 'parent', assignee => 'assignee', pta => 'pta',
        national_entry_date => 'national-entry-date',
      );
      foreach my $att ( sort keys %app_att_tbl ) {
        $twsappnode->setAttribute( $app_att_tbl{ $att }, $twsapp->{ $att } )
          if $twsapp->{ $att };
      }

      if( $twsapp->{inventors} ) {
        my $invs = XML::LibXML::Element->new('inventors');
        foreach my $inv ( @{ $twsapp->{inventors} } ) {
          my $in = XML::LibXML::Element->new('inventor');
          $in->appendText( $inv );
          $invs->addChild( $in );
        }
        $twsappnode->addChild( $invs );
      }

      if( $twsapp->{priority} ) {
        my $pr = XML::LibXML::Element->new('priority');
        foreach my $att ( sort keys %docid_att_tbl ) {
          $pr->setAttribute( $docid_att_tbl{ $att }, $twsapp->{priority}->{ $att } )
            if $twsapp->{priority}->{ $att };
        }
        $twsappnode->addChild( $pr );
      }
      if( $twsapp->{priority_original} ) {
        my $pr = XML::LibXML::Element->new('priority-original');
        foreach my $att ( sort keys %docid_att_tbl ) {
          $pr->setAttribute( $docid_att_tbl{ $att }, $twsapp->{priority_original}->{ $att } )
            if $twsapp->{priority_original}->{ $att };
        }
        $twsappnode->addChild( $pr );
      }
      if( $twsapp->{publications} ) {
        my $pubs = XML::LibXML::Element->new('publications');
        foreach my $pub ( @{ $twsapp->{publications} } ) {
          my $pubnode = XML::LibXML::Element->new('publication');
          foreach my $att ( sort keys %docid_att_tbl ) {
            $pubnode->setAttribute( $docid_att_tbl{ $att }, $pub->{ $att } )
              if $pub->{ $att };
          }
          $pubs->addChild( $pubnode );
        }
        $twsappnode->addChild( $pubs );
      }
      if( $twsapp->{grant} ) {
        my $grantnode = XML::LibXML::Element->new('grant');
        foreach my $att ( sort keys %docid_att_tbl ) {
          $grantnode->setAttribute( $docid_att_tbl{ $att }, $twsapp->{grant}->{ $att } )
            if $twsapp->{grant}->{ $att };
        }
        $twsappnode->addChild( $grantnode );
      }
      if( $twsapp->{national_grant} ) {
        my $grantnode = XML::LibXML::Element->new('national-grant');
        foreach my $att ( sort keys %docid_att_tbl ) {
          $grantnode->setAttribute( $docid_att_tbl{ $att }, $twsapp->{national_grant}->{ $att } )
            if $twsapp->{national_grant}->{ $att };
        }
        $twsappnode->addChild( $grantnode );
      }
      if( $twsapp->{titles} ) {
        my $ttlsnode = XML::LibXML::Element->new('titles');
        foreach my $ttl ( @{ $twsapp->{titles} } ) {
          my $ttlnode = XML::LibXML::Element->new('title');
          $ttlnode->setAttribute( 'lang', $ttl->{lang} );
          $ttlnode->appendText( $ttl->{title} );
          $ttlsnode->addChild( $ttlnode );
        }
        $twsappnode->addChild( $ttlsnode );
      }
      if( $twsapp->{simple_family} ) {
        my $sfamnode = XML::LibXML::Element->new('simple-family');
        foreach my $ucid ( @{ $twsapp->{simple_family} } ) {
          my $sfm = XML::LibXML::Element->new('simple-family-member');
          $sfm->setAttribute('ucid', $ucid);
          $sfamnode->addChild( $sfm );
        }
        $twsappnode->addChild( $sfamnode );
      }
      if( $twsapp->{extended_family} ) {
        my $efamnode = XML::LibXML::Element->new('extended-family');
        foreach my $ucid ( @{ $twsapp->{extended_family} } ) {
          my $efm = XML::LibXML::Element->new('extended-family-member');
          $efm->setAttribute('ucid', $ucid);
          $efamnode->addChild( $efm );
        }
        $twsappnode->addChild( $efamnode );
      }
      if( $twsapp->{classifications} ) {
        my $clsnode = XML::LibXML::Element->new('classifications');
        foreach my $cstruct ( @{ $twsapp->{classifications} } ) {
#$c->log->debug( Dumper( $cstruct ) );
          foreach my $type ( sort keys %{ $cstruct } ) {
#$c->log->debug( Dumper( $type ) );
            my $tc = XML::LibXML::Element->new('classification');
            $tc->setAttribute( 'type', $type );
            foreach my $data ( sort @{ $cstruct->{$type} } ) {
              my $cls = XML::LibXML::Element->new('text');
              $cls->appendText( $data );
              $tc->addChild( $cls );
            }
            $clsnode->addChild( $tc );
          }
        }
        $twsappnode->addChild( $clsnode );
      }
      if( $twsapp->{claims_hierarchy} ) {
        my $clmh = XML::LibXML::Element->new('claims-hierarchy');
        foreach my $clm ( @{ $twsapp->{claims_hierarchy} } ) {
          my $cl = XML::LibXML::Element->new('claim');
          $cl->setAttribute('num', $clm->{num} );
          $cl->setAttribute('type', $clm->{type} );
          if( scalar( @{ $clm->{refs} } ) ) {
            foreach my $clmref ( @{ $clm->{refs} } ) {
              my $clmr = XML::LibXML::Element->new('claim-ref');
              $clmr->setAttribute('num', $clmref);
              $cl->addChild( $clmr );
            }
          }
          $clmh->addChild( $cl );
        }
        $twsappnode->addChild( $clmh );
      }
      ### XML Text
      my @xml_members = ('abstract_en', 'abstract_original', 'claims_en', 'claims_original', 'description_en', 'description_original' );
      foreach my $member ( @xml_members ) {
        if( $twsapp->{$member} ) {
          # just text ...
          my $node = XML::LibXML::Element->new( $member );

          my $p = XML::LibXML->new( huge=>1 );
          my $frag = $p->parse_string( $twsapp->{$member} );
          $node->appendChild( $frag->documentElement() );
          $twsappnode->appendChild( $node );
#          $node->addChild( $frag->documentElement() );
#          $twsappnode->addChild( $node );
#if( $member eq 'description_en' ) {
  # all still containing namespaces ACWS-103
#  $c->log->debug( $twsapp->{$member} );
#  $c->log->debug( $node->toString(1) );
#}
        }
      }
      $tws->addChild( $twsappnode );
    }
    $root->addChild( $tws );
  }
  $c->response->output( $dom->toString(1) );
  return 1;
#      $tws->addChild( $appnode );
#    }
#    $root->addChild( $tws );
#  }
#  $c->response->output( $dom->toString(1) );
#  return 1;
}

__PACKAGE__->meta->make_immutable;
1;

