#!/usr/bin/env perl

use strict;
use warnings;
use LWP::UserAgent;
use HTTP::Headers;
use Test::More;
use Data::Dumper;

use lib './t';

use TestConfig;

my $tests = 0;

# 3 tests
$tests += 3;
sub _test_users_list {
  my $cfg = shift;
  my $mech = shift;
  my $res = $mech->get( 'http://localhost/admin/users/list' , 'Requested page');

  # web server status
  ok( $mech->status == 200, 'server response status' );

  # request status
  my $obj = $cfg->json_encoder->decode( $res->content );
  ok( $obj->{status} && $obj->{status} eq 'success' );

  # test user present
  my $id;
  my $ok = 0; # assume the worst
  foreach my $user ( @{$obj->{users}} ) {
    if( $user->{user_name} eq $cfg->user ) {
      $id = $user->{user_id};
      $ok = 1;
    }
  }
  ok($ok, 'exists: configured test user');
  return $id;
}

$tests += 3;
sub _test_users_info {
  my $cfg = shift;
  my $mech = shift;
  my $id = shift;

  my $res;
  eval { $res = $mech->get( "http://localhost/admin/users/info?user_id=$id" , 'Requested page'); };

  # web server status
  ok( $mech->status == 200, 'server response status' );

  # request status
  my $obj = $cfg->json_encoder->decode( $res->content );
  ok( $obj->{status} && $obj->{status} eq 'success' );
  ok( $obj->{user}->{user_name} eq $cfg->user );
}

$tests += 3;
sub _test_users_add {
  my $cfg = shift;
  my $mech = shift;

  my $res;
  eval { $res = $mech->get( "http://localhost/admin/users/add?user_name=__test" , 'Requested page'); };

  # web server status
  ok( $mech->status == 200, 'server response status' );

  # request status
  my $obj = $cfg->json_encoder->decode( $res->content );
  ok( $obj->{status} && $obj->{status} eq 'success' );
  ok( $obj->{user} && $obj->{user}->{user_id} > 0 );
  return $obj->{user}->{user_id};
}

$tests += 3;
sub _test_users_update {
  my $cfg = shift;
  my $mech = shift;
  my $id = shift;

  my $res;
  eval { $res = $mech->get( "http://localhost/admin/users/update?user_id=$id&user_name=__test_updated" , 'Requested page'); };

  # web server status
  ok( $mech->status == 200, 'server response status' );

  # request status
  my $obj = $cfg->json_encoder->decode( $res->content );
  ok( $obj->{status} && $obj->{status} eq 'success' );

  ok( $obj->{user}->{user_name} eq '__test_updated' );
}

$tests += 3;
sub _test_users_delete {
  my $cfg = shift;
  my $mech = shift;
  my $id = shift;

  my $res;
  eval { $res = $mech->get( "http://localhost/admin/users/delete?user_id=$id" , 'Requested page'); };

  # web server status
  ok( $mech->status == 200, 'server response status' );

  # request status
  my $obj = $cfg->json_encoder->decode( $res->content );
  ok( $obj->{status} && $obj->{status} eq 'success' );
  ok( $obj->{deleted} == 1 );
}


###########
my ($cfg, $mech);

eval {
  $cfg = TestConfig->new();
  $mech = $cfg->mech_agent;
};

plan $@ ? ( skip_all => 'Failed to load TestConfig' ) :  ( tests => $tests );

# returns user_id of configuration test user
my $id = _test_users_list( $cfg, $mech );

_test_users_info( $cfg, $mech, $id );

# add user
my $newid = _test_users_add( $cfg, $mech );

# update user
_test_users_update( $cfg, $mech, $newid );

# delete user
_test_users_delete( $cfg, $mech, $newid );

