#!/usr/bin/env perl

use strict;
use warnings;
use LWP::UserAgent;
use HTTP::Headers;
use Test::More;
use Data::Dumper;

use lib './t';

use TestConfig;

my $tests = 0;

# 3 tests
$tests += 3;
sub _test_groups_list {
  my $cfg = shift;
  my $mech = shift;
  my $res = $mech->get( 'http://localhost/admin/groups/list' , 'Requested page');

  # web server status
  ok( $mech->status == 200, 'server response status' );

  # request status
  my $obj = $cfg->json_encoder->decode( $res->content );
  ok( $obj->{status} && $obj->{status} eq 'success' );

  # test group present
  my $id;
  my $ok = 0; # assume the worst
  foreach my $group ( @{$obj->{groups}} ) {
    if( $group->{group_name} eq $cfg->group ) {
      $id = $group->{group_id};
      $ok = 1;
    }
  }
  ok($ok, 'exists: configured test group');
  return $id;
}

$tests += 3;
sub _test_groups_info {
  my $cfg = shift;
  my $mech = shift;
  my $id = shift;

  my $res;
  eval { $res = $mech->get( "http://localhost/admin/groups/info?group_id=$id" , 'Requested page'); };

  # web server status
  ok( $mech->status == 200, 'server response status' );

  # request status
  my $obj = $cfg->json_encoder->decode( $res->content );
  ok( $obj->{status} && $obj->{status} eq 'success' );
  ok( $obj->{group}->{group_name} eq $cfg->group );
}

$tests += 3;
sub _test_groups_add {
  my $cfg = shift;
  my $mech = shift;

  my $res;
  eval { $res = $mech->get( "http://localhost/admin/groups/add?group_name=__test" , 'Requested page'); };

  # web server status
  ok( $mech->status == 200, 'server response status' );

  # request status
  my $obj = $cfg->json_encoder->decode( $res->content );
  ok( $obj->{status} && $obj->{status} eq 'success' );
  ok( $obj->{group} && $obj->{group}->{group_id} > 0 );


  return $obj->{group}->{group_id};
}

$tests += 3;
sub _test_groups_update {
  my $cfg = shift;
  my $mech = shift;
  my $id = shift;

  my $res;
  eval { $res = $mech->get( "http://localhost/admin/groups/update?group_id=$id&group_name=__test_updated" , 'Requested page'); };

  # web server status
  ok( $mech->status == 200, 'server response status' );

  # request status
  my $obj = $cfg->json_encoder->decode( $res->content );
  ok( $obj->{status} && $obj->{status} eq 'success' );

  ok( $obj->{group}->{group_name} eq '__test_updated' );
}

$tests += 3;
sub _test_groups_delete {
  my $cfg = shift;
  my $mech = shift;
  my $id = shift;

  my $res;
  eval { $res = $mech->get( "http://localhost/admin/groups/delete?group_id=$id" , 'Requested page'); };

  # web server status
  ok( $mech->status == 200, 'server response status' );

  # request status
  my $obj = $cfg->json_encoder->decode( $res->content );
  ok( $obj->{status} && $obj->{status} eq 'success' );
  ok( $obj->{deleted} == 1 );
}


###########
my ($cfg, $mech);

eval {
  $cfg = TestConfig->new();
  $mech = $cfg->mech_agent;
};

plan $@ ? ( skip_all => 'Failed to load TestConfig' ) :  ( tests => $tests );

# returns group_id of configuration group
my $id = _test_groups_list( $cfg, $mech );

_test_groups_info( $cfg, $mech, $id );

# add user
my $newid = _test_groups_add( $cfg, $mech );

# update user
_test_groups_update( $cfg, $mech, $newid );

# delete user
_test_groups_delete( $cfg, $mech, $newid );

