#!/usr/bin/env perl

use strict;
use warnings;
use LWP::UserAgent;
use HTTP::Headers;
use Test::More;
use Data::Dumper;

use lib './t';

use TestConfig;

my $tests = 0;

# 3 tests
$tests += 3;
sub _test_privileges_list {
  my $cfg = shift;
  my $mech = shift;
  my $res = $mech->get( 'http://localhost/admin/privileges/list' , 'Requested page');

  # web server status
  ok( $mech->status == 200, 'server response status' );

  # request status
  my $obj = $cfg->json_encoder->decode( $res->content );
  ok( $obj->{status} && $obj->{status} eq 'success' );

  # test privilege present
  my $id;
  my $ok = 0; # assume the worst
  foreach my $privilege ( @{$obj->{privileges}} ) {
    if( $privilege->{privilege_name} eq $cfg->privilege ) {
      $id = $privilege->{privilege_id};
      $ok = 1;
    }
  }
  ok($ok, 'exists: configured test privilege');
  return $id;
}

$tests += 3;
sub _test_privileges_info {
  my $cfg = shift;
  my $mech = shift;
  my $id = shift;

  my $res;
  eval { $res = $mech->get( "http://localhost/admin/privileges/info?privilege_id=$id" , 'Requested page'); };

  # web server status
  ok( $mech->status == 200, 'server response status' );

  # request status
  my $obj = $cfg->json_encoder->decode( $res->content );
  ok( $obj->{status} && $obj->{status} eq 'success' );
  ok( $obj->{privilege}->{privilege_name} eq $cfg->privilege );
}


###########
my ($cfg, $mech);

eval {
  $cfg = TestConfig->new();
  $mech = $cfg->mech_agent;
};

plan $@ ? ( skip_all => 'Failed to load TestConfig' ) :  ( tests => $tests );

# returns privilege_id of configuration group
my $id = _test_privileges_list( $cfg, $mech );

_test_privileges_info( $cfg, $mech, $id );

